﻿using System;

namespace Sencha.Serialization.Bson.Exceptions
{
	[Serializable]
	sealed class UnexpectedBsonToken : Sencha.Serialization.Exceptions.JsonSerializationException
	{
		public UnexpectedBsonToken(IJsonReader reader, params BsonToken[] expectedTokens) :
			base(Loc.GetErrorString("Exception_UnexpectedToken", reader.IsEndOfStream() ? JsonToken.EndOfStream : reader.Token, JoinTokens(expectedTokens)), Sencha.Serialization.Exceptions.ErrorCode.UnexpectedToken, reader)
		{
			if (reader == null)
				throw new ArgumentNullException("reader");
			if (expectedTokens == null)
				throw new ArgumentNullException("expectedTokens");
			System.Diagnostics.Contracts.Contract.EndContractBlock();
		}
		protected UnexpectedBsonToken(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context)
			: base(info, context)
		{
		}

		private static object JoinTokens(BsonToken[] expectedTokens)
		{
			if (expectedTokens == null)
				throw new ArgumentNullException("expectedTokens");
			System.Diagnostics.Contracts.Contract.EndContractBlock();

			if (expectedTokens.Length == 0)
				return "<no tokens>";
#if NET40
			return String.Join(", ", expectedTokens);
#else
			var tokens = Array.ConvertAll(expectedTokens, c => c.ToString());
			return String.Join(", ", tokens);
#endif
		}
	}
}
