﻿using System;


namespace Sencha.Serialization.Bson.Serializers
{
	public class BsonMaxKeySerializer : TypeSerializer<BsonMaxKey>
	{
		public override BsonMaxKey Deserialize(IJsonReader reader)
		{
			if (reader == null)
				throw new ArgumentNullException("reader");

			if (reader is IBsonReader)
				return this.DeserializeBson(reader as IBsonReader);

			reader.ReadObjectBegin();
			reader.ReadMember<int>("$maxkey");
			reader.ReadObjectEnd(false);

			return BsonMaxKey.Value;

		}
		private BsonMaxKey DeserializeBson(IBsonReader reader)
		{
			if (reader.Token != BsonToken.MaxKey)
				throw new Exceptions.UnexpectedBsonToken(reader, BsonToken.MaxKey);

			return BsonMaxKey.Value;
		}
		public override void Serialize(IJsonWriter writer, BsonMaxKey value)
		{
			if (writer == null)
				throw new ArgumentNullException("writer");

			if (value == null)
			{
				writer.WriteNull();
				return;
			}

			if (writer is IBsonWriter)
			{
				this.SerializeBson(writer as IBsonWriter, value);
				return;
			}

			writer.WriteObjectBegin();
			writer.WriteMember<int>("$maxkey", 1);
			writer.WriteObjectEnd();
		}
		private void SerializeBson(IBsonWriter writer, BsonMaxKey value)
		{
			writer.Write(BsonToken.MaxKey, (object)null);
		}
	}
}
