﻿using System;

namespace Sencha.Serialization.Bson.Serializers
{
	public class BsonTimestampSerializer : TypeSerializer<BsonTimestamp>
	{
		public override BsonTimestamp Deserialize(IJsonReader reader)
		{
			if (reader == null)
				throw new ArgumentNullException("reader");

			if (reader is IBsonReader)
				return this.DeserializeBson(reader as IBsonReader);

			reader.ReadObjectBegin();
			var timeStampString = reader.ReadMember<string>("$timestamp");
			reader.ReadObjectEnd(false);

			var timeStampValue = long.Parse(timeStampString);
			var value = new BsonTimestamp(timeStampValue);
			return value;
		}
		private BsonTimestamp DeserializeBson(IBsonReader reader)
		{
			if (reader.Token != BsonToken.Timestamp)
				throw new Exceptions.UnexpectedBsonToken(reader, BsonToken.Timestamp);

			var objectId = (BsonTimestamp)reader.RawValue;
			return objectId;
		}
		public override void Serialize(IJsonWriter writer, BsonTimestamp value)
		{
			if (writer == null)
				throw new ArgumentNullException("writer");

			if (writer is IBsonWriter)
			{
				this.SerializeBson(writer as IBsonWriter, value);
				return;
			}

			writer.WriteObjectBegin();
			writer.WriteMember<long>("$timestamp", value.Raw);
			writer.WriteObjectEnd();
		}
		private void SerializeBson(IBsonWriter writer, BsonTimestamp value)
		{
			writer.Write(BsonToken.Timestamp, value);
		}
	}
}
