﻿using System;

namespace Sencha.Serialization
{
	public sealed class JsonStringReader : JsonReaderBase
	{
		private string m_jsonString;
		private int m_position;

		public JsonStringReader(string jsonString)
			: this(jsonString, new AdHocSerializer(), DefaultBufferSize)
		{

		}
		public JsonStringReader(string jsonString, int bufferSize = DefaultBufferSize)
			: this(jsonString, new AdHocSerializer(), bufferSize)
		{

		}
		public JsonStringReader(string jsonString, ISerializationContext context, int bufferSize = DefaultBufferSize)
			: base(context, bufferSize)
		{
			if (jsonString == null)
				throw new ArgumentNullException("jsonString");
			System.Diagnostics.Contracts.Contract.EndContractBlock();

			m_jsonString = jsonString;
			m_position = 0;
		}

		protected override int FillBuffer(char[] buffer, int index)
		{
			if (buffer == null)
				throw new ArgumentNullException("buffer");
			if (index < 0 || index >= buffer.Length)
				throw new ArgumentOutOfRangeException("index");
			System.Diagnostics.Contracts.Contract.EndContractBlock();

			var block = Math.Min(m_jsonString.Length - m_position, buffer.Length - index);
			if (block <= 0)
				return index;

			m_jsonString.CopyTo(m_position, buffer, index, block);

			m_position += block;

			return index + block;
		}
	}
}
