﻿using Sencha.Serialization.Contracts;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for DataContractTest and is intended
	///to contain all DataContractTest Unit Tests
	///</summary>
	[TestClass()]
	public class DataContractTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for DataContract Constructor
		///</summary>
		[TestMethod()]
		public void DataContractConstructorTest()
		{
			Type rootType = typeof(ConsoleColor);
			MetadataInterpreter interpreter = DataContractInterpreter.DataContract;
			MetadataDescriber descriptor = MetadataDescriber.ReflectionPublicOnly;
			DataContract target = new DataContract(rootType, interpreter, descriptor);
		}

		/// <summary>
		///A test for AddKnownType
		///</summary>
		[TestMethod()]
		public void AddKnownTypeTest()
		{
			Type rootType = typeof(ConsoleColor);
			MetadataInterpreter interpreter = MetadataInterpreter.DataContract;
			MetadataDescriber descriptor = MetadataDescriber.ReflectionPublicOnly;
			DataContract target = new DataContract(rootType, interpreter, descriptor);

			Type type = typeof(RankException);
			TypeDescription actual;
			actual = target.AddKnownType(type);

			Assert.IsNotNull(actual);
			Assert.AreEqual(type, actual.GetClrType());
		}

		/// <summary>
		///A test for AddKnownTypes
		///</summary>
		[TestMethod()]
		public void AddKnownTypesListTest()
		{
			Type rootType = typeof(ConsoleColor);
			MetadataInterpreter interpreter = MetadataInterpreter.DataContract;
			MetadataDescriber descriptor = MetadataDescriber.ReflectionPublicOnly;
			DataContract target = new DataContract(rootType, interpreter, descriptor);

			IList<Type> types = new Type[] { typeof(string), typeof(EqualityComparer<Exception>), typeof(DBNull), typeof(Version), typeof(IntPtr) };
			TypeDescription[] actual;
			actual = target.AddKnownTypes(types);

			Assert.IsNotNull(actual);
			var actualTypes = Array.ConvertAll(actual, t => t.GetClrType());
			Assert.AreEqual(types.Count, actualTypes.Length, "arrays length did't match");
			for (int i = 0; i < types.Count; i++)
			{
				Assert.AreEqual(types[i], actualTypes[i]);
			}
		}

		/// <summary>
		///A test for AddKnownTypes
		///</summary>
		[TestMethod()]
		public void AddKnownTypesArrayTest()
		{
			Type rootType = typeof(ConsoleColor);
			MetadataInterpreter interpreter = MetadataInterpreter.DataContract;
			MetadataDescriber descriptor = MetadataDescriber.ReflectionPublicOnly;
			DataContract target = new DataContract(rootType, interpreter, descriptor);

			Type[] types = new Type[] { typeof(string), typeof(EqualityComparer<Exception>), typeof(DBNull), typeof(Version), typeof(IntPtr) };
			TypeDescription[] actual;
			actual = target.AddKnownTypes(types);
			Assert.IsNotNull(actual);

			var actualTypes = Array.ConvertAll(actual, t => t.GetClrType());
			Assert.AreEqual(types.Length, actualTypes.Length, "arrays length did't match");
			for (int i = 0; i < types.Length; i++)
			{
				Assert.AreEqual(types[i], actualTypes[i]);
			}
		}

		/// <summary>
		///A test for GetKnownType
		///</summary>
		[TestMethod()]
		public void GetKnownTypeTest()
		{
			Type rootType = typeof(ConsoleColor);
			MetadataInterpreter interpreter = MetadataInterpreter.DataContract;
			MetadataDescriber descriptor = MetadataDescriber.ReflectionPublicOnly;
			DataContract target = new DataContract(rootType, interpreter, descriptor);

			Type type = typeof(ConsoleColor);
			TypeDescription actual;
			actual = target.GetKnownType(type);

			Assert.IsNotNull(actual);
			Assert.AreEqual(type, actual.GetClrType());
		}

		/// <summary>
		///A test for TryGetKnownType
		///</summary>
		[TestMethod()]
		public void TryGetKnownTypeTest()
		{
			Type rootType = typeof(ConsoleColor);
			MetadataInterpreter interpreter = MetadataInterpreter.DataContract;
			MetadataDescriber descriptor = MetadataDescriber.ReflectionPublicOnly;
			DataContract target = new DataContract(rootType, interpreter, descriptor);

			Type type = typeof(ConsoleColor);
			TypeDescription result = null;
			TypeDescription resultExpected = target.Root;
			bool expected = true;
			bool actual;
			actual = target.TryGetKnownType(type, out result);

			Assert.IsNotNull(actual);
			Assert.AreEqual(expected, actual);
			Assert.AreSame(resultExpected, result);
		}
	}
}
