﻿using Sencha.Serialization.Contracts;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.Serialization;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for MemberInfoTest and is intended
	///to contain all MemberInfoTest Unit Tests
	///</summary>
	[TestClass()]
	public class MemberInfoTest
	{
		[DataContract]
		public class KeyValue<K, V>
		{
			[DataMember]
			public K Key;
			[DataMember(Name = "ValueField"), TypeConverter(typeof(Int16Converter))]
			public V Value { get; set; }
		}

		public class MyClassA
		{
			public int Field { get { return 0; } }
		}

		[DataContract]
		public class MyClassOrderedMembers
		{
			[DataMember]
			public int Field1 { get { return 0; } }
			[DataMember]
			public int Field2 { get { return 0; } }
			[DataMember]
			public int Field3 { get { return 0; } }
			[DataMember]
			public int Field4 { get { return 0; } }
			[DataMember]
			public int Field5 { get { return 0; } }
		}


		private DataContract SampleContract = new DataContract(typeof(KeyValue<int, string>));
		private System.Reflection.MemberInfo SampleContractKeyMember { get { return SampleContract.Root.GetClrType().GetField("Key"); } }
		private System.Reflection.MemberInfo SampleContractValueMember { get { return SampleContract.Root.GetClrType().GetProperty("Value"); } }

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for MemberInfo Constructor
		///</summary>
		[TestMethod()]
		public void MemberInfoConstructorTest()
		{
			TypeDescription declaringType = SampleContract.Root;
            System.Reflection.MemberInfo metadataMember = SampleContractKeyMember;
			MemberDescription target = new MemberDescription(declaringType, metadataMember);
		}

		private MemberDescription GetTestMemberInfo()
		{
			TypeDescription declaringType = SampleContract.Root;

			return declaringType.Members[0];
		}


		/// <summary>
		///A test for CompareTo
		///</summary>
		[TestMethod()]
		public void CompareToTest()
		{
			TypeDescription declaringType = SampleContract.Root;
			MemberDescription target = declaringType.Members[0];
			MemberDescription other = declaringType.Members[1];
			int expected = target.Name.CompareTo(other.Name);
			int actual;
			actual = target.CompareTo((object)other);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Equals
		///</summary>
		[TestMethod()]
		public void EqualsTest()
		{
			TypeDescription declaringType = SampleContract.Root;
			MemberDescription target = declaringType.Members[0];
			MemberDescription other = declaringType.Members[0];
			bool expected = true;
			bool actual;
			actual = target.Equals((object)other);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for GetHashCode
		///</summary>
		[TestMethod()]
		public void GetHashCodeTest()
		{
			MemberDescription target = GetTestMemberInfo();
			int actual;
			actual = target.GetHashCode();
		}

		/// <summary>
		///A test for ToString
		///</summary>
		[TestMethod()]
		public void ToStringTest()
		{
			var target = this.GetTestMemberInfo();
			string actual;
			actual = target.ToString();
			Assert.IsNotNull(actual);
		}

		/// <summary>
		///A test for Attributes
		///</summary>
		[TestMethod()]
		public void AttributesTest()
		{
			TypeDescription declaringType = SampleContract.Root;
			var target = declaringType.Members.Find("ValueField");

			int expected = 2;
			IList<Attribute> actual;
			actual = target.Attributes;

			Assert.IsNotNull(actual);
			Assert.AreEqual(expected, actual.Count);
		}

        ///// <summary>
        /////A test for Converter
        /////</summary>
        //[TestMethod()]
        //public void ConverterTest()
        //{
        //    TypeInfo declaringType = SampleContract.Root;
        //    var target = declaringType.Members.Find("ValueField");
        //    var expected = typeof(Int16Converter);
        //    Type actual;
        //    actual = target.ConverterType;

        //    Assert.IsNotNull(actual);
        //    Assert.AreEqual(expected, actual);
        //}

        ///// <summary>
        /////A test for Converter
        /////</summary>
        //[TestMethod()]
        //public void ConverterNullTest()
        //{
        //    TypeInfo declaringType = SampleContract.Root;
        //    var target = declaringType.Members.Find("Key");
        //    Type actual;
        //    actual = target.ConverterType;

        //    Assert.IsNull(actual);
        //}

		/// <summary>
		///A test for DeclaringType
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeclaringTypeTest()
		{
			TypeDescription declaringType = SampleContract.Root;
			var member = declaringType.Members.Find("Key");
            MemberDescription_Accessor target = new MemberDescription_Accessor(new PrivateObject(member));
			TypeDescription expected = declaringType;
			TypeDescription actual;
			actual = target.DeclaringType;

			Assert.AreSame(expected, actual);
		}

        ///// <summary>
        /////A test for IsReadOnly
        /////</summary>
        //[TestMethod()]
        //public void IsReadOnlyTest()
        //{
        //    TypeInfo declaringType = SampleContract.Root;
        //    var target = declaringType.Members.Find("Key");
        //    bool expected = false;
        //    bool actual;
        //    actual = target.IsReadOnly;
        //    Assert.AreEqual(expected, actual);
        //}

        ///// <summary>
        /////A test for IsReadOnly
        /////</summary>
        //[TestMethod()]
        //public void IsReadOnlyNoSetterTest()
        //{
        //    TypeInfo declaringType = new DataContract(typeof(MyClassA)).Root;
        //    var target = declaringType.Members.Find("Field");
        //    bool expected = true;
        //    bool actual;
        //    actual = target.IsReadOnly;
        //    Assert.AreEqual(expected, actual);
        //}

        ///// <summary>
        /////A test for MemberType
        /////</summary>
        //[TestMethod()]
        //public void MemberTypeTest()
        //{
        //    TypeInfo declaringType = new DataContract(typeof(MyClassA)).Root;
        //    var target = declaringType.Members.Find("Field");
        //    Type expected = typeof(int);
        //    Type actual;
        //    actual = target.MemberType;
        //    Assert.AreEqual(expected, actual);
        //}


        ///// <summary>
        /////A test for MemberType
        /////</summary>
        //[TestMethod()]
        //public void MemberTypeGenericTest()
        //{
        //    TypeInfo declaringType = new DataContract(typeof(KeyValue<ConsoleColor, int>)).Root;
        //    var target = declaringType.Members.Find("Key");
        //    Type expected = typeof(ConsoleColor);
        //    Type actual;
        //    actual = target.MemberType;
        //    Assert.AreEqual(expected, actual);
        //}

        ///// <summary>
        /////A test for MemberTypeInfo
        /////</summary>
        //[TestMethod()]
        //public void MemberTypeInfoTest()
        //{
        //    var contract = new DataContract(typeof(KeyValue<ConsoleColor, int>));
        //    TypeInfo declaringType = contract.Root;
        //    var target = declaringType.Members.Find("Key");
        //    TypeInfo expected = contract.GetKnownType(typeof(ConsoleColor));
        //    TypeInfo actual;
        //    actual = target.MemberTypeInfo;
        //    Assert.AreEqual(expected, actual);
        //}

		/// <summary>
		///A test for Name
		///</summary>
		[TestMethod()]
		public void NameTest()
		{
			TypeDescription declaringType = SampleContract.Root;
			var target = declaringType.Members.Find("ValueField");
			string expected = "ValueField";
			string actual;
			actual = target.Name;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Ordinal
		///</summary>
		[TestMethod()]
		public void OrdinalTest()
		{
			var contract = new DataContract(typeof(MyClassOrderedMembers));
			TypeDescription declaringType = contract.Root;
			var target = declaringType.Members.Find("Field2");
			int expected = 1;
			int actual;
			actual = target.Ordinal;
			Assert.AreEqual(expected, actual);
		}
	}
}
