﻿using Sencha.Serialization.Contracts;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.Serialization;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for TypeInfoTest and is intended
	///to contain all TypeInfoTest Unit Tests
	///</summary>
	[TestClass()]
	public class TypeInfoTest
	{
		#region test classes
		[Serializable]
		public class MyClassA
		{
			public int MyMember12 { get; private set; }
		}
		[Serializable]
		public class MyClassB : MyClassA
		{
			public int MyMember { get; private set; }
			public int MyMember2 { private get; set; }
		}
		[TypeConverter(typeof(Int16Converter)), TypeSerializer(typeof(Serializers.ArraySerializer<int[]>))]
		public class MyClassC : MyClassA
		{

		}
		[DataContractAttribute(Name = "RefedinedNameB")]
		public class MyClassD : IJsonSerializable
		{
			#region IJsonSerializable Members

            public void ReadJson(IJsonReader reader)
			{
				throw new NotImplementedException();
			}

            public void WriteJson(IJsonWriter writer)
			{
				throw new NotImplementedException();
			}

			#endregion
		}
		#endregion

		public DataContract MyClassBContract = new DataContract(typeof(MyClassB), Type.EmptyTypes, MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract MyClassCContract = new DataContract(typeof(MyClassC), Type.EmptyTypes, MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion

		/// <summary>
		///A test for BaseType
		///</summary>
		[TestMethod()]
		public void BaseTypeTest()
		{
			TypeDescription target = MyClassBContract.Root;
			TypeDescription expected = MyClassBContract.GetKnownType(typeof(MyClassA));
			TypeDescription actual;
			target.BaseType = expected;
			actual = target.BaseType;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Attributes
		///</summary>
		[TestMethod()]
		public void AttributesTest()
		{
			TypeDescription target = MyClassBContract.Root;
			IList<Type> expected = new[] { typeof(SerializableAttribute) };
			IList<Attribute> actual;
			actual = target.Attributes;

			Assert.IsNotNull(actual);
			Assert.AreEqual(expected.Count, actual.Count);
			for (int i = 0; i < actual.Count; i++)
			{
				if (!expected.Contains(actual[i].GetType()))
					Assert.Fail("Type does not contrains attribute of type " + actual[i].GetType().FullName);
			}
		}


		/// <summary>
		///A test for ToString
		///</summary>
		[TestMethod()]
		public void ToStringTest()
		{
			TypeDescription target = MyClassBContract.Root;

			string actual = target.ToString();

			Assert.IsNotNull(actual);
		}

		/// <summary>
		///A test for IsDescendantOf
		///</summary>
		[TestMethod()]
		public void IsDescendantOfTest()
		{
			TypeDescription target = MyClassBContract.Root;
			TypeDescription typeInfo = MyClassBContract.GetKnownType(typeof(MyClassA));
			bool expected = true;
			bool actual;
			actual = target.IsDescendantOf(typeInfo);
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for IsDescendantOf
		///</summary>
		[TestMethod()]
		public void IsDescendantOfObjectTest()
		{
			TypeDescription target = MyClassBContract.Root;
			TypeDescription typeInfo = MyClassBContract.GetKnownType(typeof(object));
			bool expected = true;
			bool actual;
			actual = target.IsDescendantOf(typeInfo);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsDescendantOf
		///</summary>
		[TestMethod()]
		public void NegativeIsDescendantOfObjectTest()
		{
			TypeDescription target = MyClassBContract.Root;
			TypeDescription typeInfo = MyClassBContract.GetKnownType(typeof(int));
			bool expected = false;
			bool actual;
			actual = target.IsDescendantOf(typeInfo);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for TypeInfo Constructor
		///</summary>
		[TestMethod()]
		public void TypeInfoConstructorTest()
		{
			Type contractedType = typeof(string);
			TypeDescription target = new TypeDescription(contractedType);
		}

		/// <summary>
		///A test for GetHashCode
		///</summary>
		[TestMethod()]
		public void GetHashCodeTest()
		{
			TypeDescription target = MyClassBContract.Root;
			int actual;
			actual = target.GetHashCode();
		}

		/// <summary>
		///A test for Equals
		///</summary>
		[TestMethod()]
		public void EqualsTest()
		{
			var contractA = new DataContract(typeof(MyClassA));
			var contractB = new DataContract(typeof(MyClassB));

			TypeDescription target = contractA.GetKnownType(typeof(MyClassA));
			TypeDescription other = contractB.GetKnownType(typeof(MyClassA));
			bool expected = true;
			bool actual;
			actual = target.Equals(other);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Equals
		///</summary>
		[TestMethod()]
		public void NegativeEqualsTest()
		{
			var contractA = new DataContract(typeof(MyClassA));
			var contractB = new DataContract(typeof(MyClassB));

			TypeDescription target = contractA.GetKnownType(typeof(MyClassA));
			TypeDescription other = contractB.GetKnownType(typeof(MyClassB));
			bool expected = false;
			bool actual;
			actual = target.Equals(other);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Equals
		///</summary>
		[TestMethod()]
		public void EqualsObjectTest()
		{
			var contractA = new DataContract(typeof(MyClassA));
			var contractB = new DataContract(typeof(MyClassB));

			TypeDescription target = contractA.GetKnownType(typeof(MyClassA));
			object other = contractB.GetKnownType(typeof(MyClassA));
			bool expected = true;
			bool actual;
			actual = target.Equals(other);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Equals
		///</summary>
		[TestMethod()]
		public void NegativeEqualsObjectTest()
		{
			var contractA = new DataContract(typeof(MyClassA));
			var contractB = new DataContract(typeof(MyClassB));

			TypeDescription target = contractA.GetKnownType(typeof(MyClassA));
			object other = 94;
			bool expected = false;
			bool actual;
			actual = target.Equals(other);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Equals
		///</summary>
		[TestMethod()]
		public void EqualsObjectNullTest()
		{
			var contractA = new DataContract(typeof(MyClassA));
			var contractB = new DataContract(typeof(MyClassB));

			TypeDescription target = contractA.GetKnownType(typeof(MyClassA));
			object other = null;
			bool expected = false;
			bool actual;
			actual = target.Equals(other);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for GetClrType
		///</summary>
		[TestMethod()]
		public void GetClrTypeTest()
		{

			TypeDescription target = MyClassBContract.Root;
			Type expected = typeof(MyClassB);
			Type actual;
			actual = target.GetClrType();
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Converter
		///</summary>
		[TestMethod()]
		public void HasConverterTest()
		{
			TypeDescription target = MyClassCContract.Root;

			var expected = true;
			bool actual;
			actual = target.IsConvertible;

			Assert.IsNotNull(actual);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Converter
		///</summary>
		[TestMethod()]
		public void ConverterNullTest()
		{
			TypeDescription target = MyClassBContract.Root;

			var expected = false;
			bool actual;
			actual = target.IsConvertible;

			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for FullName
		///</summary>
		[TestMethod()]
		public void TypeFullNameTest()
		{
			TypeDescription target = MyClassBContract.Root;
			string expected = typeof(MyClassB).FullName;
			string actual;
			actual = target.TypeFullName;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsArray
		///</summary>
		[TestMethod()]
		public void IsArrayObjectTest()
		{
			TypeDescription target = MyClassBContract.Root;
			bool expected = false;
			bool actual;
			actual = target.IsArray;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsArray
		///</summary>
		[TestMethod()]
		public void IsArrayTest()
		{
			TypeDescription target = new DataContract(typeof(MyClassB[])).Root;
			bool expected = true;
			bool actual;
			actual = target.IsArray;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsArray
		///</summary>
		[TestMethod()]
		public void IsArrayListTest()
		{
			TypeDescription target = new DataContract(typeof(List<MyClassB>)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsArray;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsBuiltIn
		///</summary>
		[TestMethod()]
		public void IsBuiltInObjectTest()
		{
			TypeDescription target = MyClassBContract.Root;
			bool expected = false;
			bool actual;
			actual = target.IsBuiltIn;
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for IsBuiltIn
		///</summary>
		[TestMethod()]
		public void IsBuiltInUntTest()
		{
			TypeDescription target = new DataContract(typeof(int)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsBuiltIn;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsDictionary
		///</summary>
		[TestMethod()]
		public void IsDictionaryGenericInterfaceTest()
		{
			TypeDescription target = new DataContract(typeof(IDictionary<string, MyClassB>)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsDictionary;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsDictionary
		///</summary>
		[TestMethod()]
		public void IsDictionaryGenericDictTest()
		{
			TypeDescription target = new DataContract(typeof(Dictionary<string, MyClassB>)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsDictionary;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsDictionary
		///</summary>
		[TestMethod()]
		public void IsDictionaryHashtableTest()
		{
			TypeDescription target = new DataContract(typeof(System.Collections.Hashtable)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsDictionary;
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for IsDictionary
		///</summary>
		[TestMethod()]
		public void IsDictionaryInterfaceTest()
		{
			TypeDescription target = new DataContract(typeof(System.Collections.IDictionary)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsDictionary;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsDictionary
		///</summary>
		[TestMethod()]
		public void IsDictionaryTest()
		{
			TypeDescription target = new DataContract(typeof(HashSet<bool>)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsDictionary;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsEnumerable
		///</summary>
		[TestMethod()]
		public void IsEnumerableHashtableTest()
		{
			TypeDescription target = new DataContract(typeof(System.Collections.Hashtable)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsEnumerable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsEnumerable
		///</summary>
		[TestMethod()]
		public void IsEnumerableArrayListTest()
		{
			TypeDescription target = new DataContract(typeof(System.Collections.ArrayList)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsEnumerable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsEnumerable
		///</summary>
		[TestMethod()]
		public void IsEnumerableGenericInterfaceTest()
		{
			TypeDescription target = new DataContract(typeof(IEnumerable<int>)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsEnumerable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsEnumerable
		///</summary>
		[TestMethod()]
		public void IsEnumerableInterfaceTest()
		{
			TypeDescription target = new DataContract(typeof(System.Collections.IEnumerable)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsEnumerable;
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for IsEnumerable
		///</summary>
		[TestMethod()]
		public void IsEnumerableArrayTest()
		{
			TypeDescription target = new DataContract(typeof(object[])).Root;
			bool expected = true;
			bool actual;
			actual = target.IsEnumerable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsEnumerable
		///</summary>
		[TestMethod()]
		public void IsEnumerableObjectTest()
		{
			TypeDescription target = new DataContract(typeof(object)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsEnumerable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsEnumerable
		///</summary>
		[TestMethod()]
		public void IsEnumerableStringTest()
		{
			TypeDescription target = new DataContract(typeof(string)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsEnumerable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsInterface
		///</summary>
		[TestMethod()]
		public void IsInterfaceObjectTest()
		{
			TypeDescription target = new DataContract(typeof(string)).Root;
			bool expected = false; // TODO: Initialize to an appropriate value
			bool actual;
			actual = target.IsInterface;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsInterface
		///</summary>
		[TestMethod()]
		public void IsInterfaceTest()
		{
			TypeDescription target = new DataContract(typeof(IEnumerable<int>)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsInterface;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsJsonSerializable
		///</summary>
		[TestMethod()]
		public void IsJsonSerializableInterfaceTest()
		{
			TypeDescription target = new DataContract(typeof(IJsonSerializable)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsJsonSerializable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsJsonSerializable
		///</summary>
		[TestMethod()]
		public void IsJsonSerializableObjectTest()
		{
			TypeDescription target = new DataContract(typeof(MyClassD)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsJsonSerializable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsNullable
		///</summary>
		[TestMethod()]
		public void IsNullableObjectTest()
		{
			TypeDescription target = new DataContract(typeof(MyClassD)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsNullable;
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for IsNullable
		///</summary>
		[TestMethod()]
		public void IsNullableIntTest()
		{
			TypeDescription target = new DataContract(typeof(int)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsNullable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsNullable
		///</summary>
		[TestMethod()]
		public void IsNullableIntNullableTest()
		{
			TypeDescription target = new DataContract(typeof(int?)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsNullable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsNullable
		///</summary>
		[TestMethod()]
		public void IsNullableGuidNullableTest()
		{
			TypeDescription target = new DataContract(typeof(Guid?)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsNullable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsNumeric
		///</summary>
		[TestMethod()]
		public void IsNumericObjectTest()
		{
			TypeDescription target = new DataContract(typeof(object)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsNumeric;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsNumeric
		///</summary>
		[TestMethod()]
		public void IsNumericIntTest()
		{
			TypeDescription target = new DataContract(typeof(int)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsNumeric;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsNumeric
		///</summary>
		[TestMethod()]
		public void IsNumericIntNullableTest()
		{
			TypeDescription target = new DataContract(typeof(int?)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsNumeric;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsObjectType
		///</summary>
		[TestMethod()]
		public void IsObjectTypeIntNullableTest()
		{
			TypeDescription target = new DataContract(typeof(int?)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsObjectType;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsObjectType
		///</summary>
		[TestMethod()]
		public void IsObjectTypeTest()
		{
			TypeDescription target = new DataContract(typeof(object)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsObjectType;
			Assert.AreEqual(expected, actual);
		}

        /// <summary>
        ///A test for IsPrimitive
        ///</summary>
        [TestMethod()]
        public void IsPrimitiveIntNullableTest()
        {
            TypeDescription target = new DataContract(typeof(int?)).Root;
            bool expected = true;
            bool actual;
            actual = target.IsPrimitive;
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for IsPrimitive
        ///</summary>
        [TestMethod()]
        public void IsPrimitiveIntTest()
        {
            TypeDescription target = new DataContract(typeof(int)).Root;
            bool expected = true;
            bool actual;
            actual = target.IsPrimitive;
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        ///A test for IsPrimitive
        ///</summary>
        [TestMethod()]
        public void IsPrimitiveEnumTest()
        {
            TypeDescription target = new DataContract(typeof(ConsoleColor)).Root;
            bool expected = false;
            bool actual;
            actual = target.IsPrimitive;
            Assert.AreEqual(expected, actual);
        }

		/// <summary>
		///A test for IsReferenceType
		///</summary>
		[TestMethod()]
		public void IsReferenceTypeEnumTest()
		{
			TypeDescription target = new DataContract(typeof(ConsoleColor)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsReferenceType;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsReferenceType
		///</summary>
		[TestMethod()]
		public void IsReferenceTypeObjectTest()
		{
			TypeDescription target = new DataContract(typeof(object)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsReferenceType;
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for IsReferenceType
		///</summary>
		[TestMethod()]
		public void IsReferenceTypeInterfaceTest()
		{
			TypeDescription target = new DataContract(typeof(IEnumerable<int>)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsReferenceType;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsSerializable
		///</summary>
		[TestMethod()]
		public void IsSerializableObjectWithAttributeTest()
		{
			TypeDescription target = new DataContract(typeof(MyClassA)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsSerializable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsSerializable
		///</summary>
		[TestMethod()]
		public void IsSerializableIntTest()
		{
			TypeDescription target = new DataContract(typeof(int)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsSerializable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsSerializable
		///</summary>
		[TestMethod()]
		public void IsSerializableObjectTest()
		{
			TypeDescription target = new DataContract(typeof(MyClassD)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsSerializable;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsTypeSerializer
		///</summary>
		[TestMethod()]
		public void IsTypeSerializerTest()
		{
			TypeDescription target = new DataContract(typeof(VersionSerializer)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsTypeSerializer;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsTypeSerializer
		///</summary>
		[TestMethod()]
		public void IsTypeSerializerIntTest()
		{
			TypeDescription target = new DataContract(typeof(int)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsTypeSerializer;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsValueType
		///</summary>
		[TestMethod()]
		public void IsValueTypeIntTest()
		{
			TypeDescription target = new DataContract(typeof(int)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsValueType;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsValueType
		///</summary>
		[TestMethod()]
		public void IsValueTypeIntNullableTest()
		{
			TypeDescription target = new DataContract(typeof(int)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsValueType;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsValueType
		///</summary>
		[TestMethod()]
		public void IsValueTypeEnumTest()
		{
			TypeDescription target = new DataContract(typeof(ConsoleColor)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsValueType;
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for IsValueType
		///</summary>
		[TestMethod()]
		public void IsValueTypeObjectTest()
		{
			TypeDescription target = new DataContract(typeof(object)).Root;
			bool expected = false;
			bool actual;
			actual = target.IsValueType;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsValueType
		///</summary>
		[TestMethod()]
		public void IsValueTypeEnumNullableTest()
		{
			TypeDescription target = new DataContract(typeof(ConsoleColor?)).Root;
			bool expected = true;
			bool actual;
			actual = target.IsValueType;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Members
		///</summary>
		[TestMethod()]
		public void MembersNoInheritanceTest()
		{

			TypeDescription target = new DataContract(typeof(MyClassA)).Root;
			int expected = 1;
			IList<MemberDescription> actual;
			actual = target.Members;
			Assert.IsNotNull(actual);
			Assert.AreEqual(expected, actual.Count);
		}

		/// <summary>
		///A test for Members
		///</summary>
		[TestMethod()]
		public void MembersInheritanceTest()
		{
			TypeDescription target = new DataContract(typeof(MyClassB)).Root;
			int expected = 2;
			IList<MemberDescription> actual;
			actual = target.Members;
			Assert.IsNotNull(actual);
			Assert.AreEqual(expected, actual.Count);
		}

		/// <summary>
		///A test for Name
		///</summary>
		[TestMethod()]
		public void NameTest()
		{
			TypeDescription target = new DataContract(typeof(MyClassD)).Root;
			string expected = "RefedinedNameB";
			string actual = target.Name;
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for GetSerializerOfType
		///</summary>
		[TestMethod()]
		public void GetTypeSerializerTest()
		{
			var contract = new DataContract(typeof(VersionSerializer));
			TypeDescription target = contract.GetKnownType(typeof(Version));
			ITypeSerializer actual;
			actual = contract.GetSerializerOfType(target);
			Assert.IsInstanceOfType(actual, typeof(VersionSerializer));
		}
	}
}
