﻿using Sencha.Serialization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Reflection;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.IO;

namespace Sencha.Serialization.Tests
{

	class JsonWriterDummy : IJsonWriter
	{
		public List<MethodInvocationInfo> Calls = new List<MethodInvocationInfo>(100);

		public JsonWriterDummy()
		{
			this.Context = new AdHocSerializer();
		}

		public ISerializationContext Context { get; private set; }
		public long CharactersWritten { get; set; }

		public void Flush()
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod()));

		}
		public void Reset()
		{
			this.CharactersWritten = 0;
		}

		public void Write(string value)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.StringLiteral, value));
			this.CharactersWritten += Convert.ToString(value).Length;
		}
		public void Write(JsonMember value)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Member, value.ToString()));
			this.CharactersWritten += Convert.ToString(value).Length;
		}
		public void Write(int number)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Number, number));
			this.CharactersWritten += Convert.ToString(number).Length;
		}
		public void Write(uint number)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Number, number));
			this.CharactersWritten += Convert.ToString(number).Length;
		}
		public void Write(long number)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Number, number));
			this.CharactersWritten += Convert.ToString(number).Length;
		}
		public void Write(ulong number)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Number, number));
			this.CharactersWritten += Convert.ToString(number).Length;
		}
		public void Write(float number)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Number, number));
			this.CharactersWritten += Convert.ToString(number).Length;
		}
		public void Write(double number)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Number, number));
			this.CharactersWritten += Convert.ToString(number).Length;
		}
		public void Write(decimal number)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Number, number));
			this.CharactersWritten += Convert.ToString(number).Length;
		}
		public void Write(bool value)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Boolean, value));
			this.CharactersWritten += Convert.ToString(value).Length;
		}
		public void Write(DateTime datetime)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), JsonToken.Boolean, datetime));
			this.CharactersWritten += Convert.ToString(datetime).Length;
		}
		public void Write(JsonToken token)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), token, null));
			this.CharactersWritten += Convert.ToString(token).Length;
		}
		public void WriteJson(string jsonString)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), jsonString));
			if (jsonString != null)
				this.CharactersWritten += jsonString.Length;
		}
		public void WriteJson(char[] jsonString, int start, int len)
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod(), jsonString, start, len));
			if (jsonString != null)
				this.CharactersWritten += len;
		}
	}
	class JsonReaderDummy : IJsonReader, IValueInfo
	{
		public List<MethodInvocationInfo> Calls = new List<MethodInvocationInfo>(100);
		public List<KeyValuePair<JsonToken, object>> Tokens = new List<KeyValuePair<JsonToken, object>>();
		public int Position = 0;

		public ISerializationContext Context { get; private set; }
		public JsonToken Token { get; private set; }
		public object RawValue { get; private set; }
		IValueInfo IJsonReader.Value { get { return this; } }
		public Type RawType
		{
			get
			{
				if (this.RawValue != null)
					return this.RawValue.GetType();
				else
					return null;
			}
		}
		public long CharactersReaded { get; set; }

		public JsonReaderDummy()
		{
			this.Context = new AdHocSerializer();
			this.Token = JsonToken.None;
		}

		public void AddToken(JsonToken token, object value)
		{
			this.Tokens.Add(new KeyValuePair<JsonToken, object>(token, value));
			if (this.Tokens.Count == 1)
			{
				this.Token = token;
				this.RawValue = value;
			}
		}
		public bool NextToken()
		{
			this.Position++;

			if (this.Position < this.Tokens.Count)
			{
				this.Token = this.Tokens[this.Position].Key;
				this.RawValue = this.Tokens[this.Position].Value;
				this.HasValue = this.Token != JsonToken.BeginArray && this.Token != JsonToken.EndOfArray && 
					this.Token != JsonToken.BeginObject && this.Token != JsonToken.EndOfObject && 
					this.Token != JsonToken.EndOfStream;
			}
			else
			{
				this.Token = JsonToken.EndOfStream;
				this.RawValue = null;
				this.HasValue = false;
			}

			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod()));
			return true;
		}
		public void Reset()
		{
			this.CharactersReaded = 0;
		}
		public bool IsEndOfStream()
		{
			this.Calls.Add(new MethodInvocationInfo(MethodBase.GetCurrentMethod()));

			return (this.Position + 1) >= this.Tokens.Count;
		}

		#region ITokenInfo Members

		public bool HasValue { get; set; }

		object IValueInfo.Raw
		{
			get { return this.RawValue; }
		}

		int IValueInfo.LineNumber
		{
			get { return 0; }
		}

		int IValueInfo.ColumnNumber
		{
			get { return 0; }
		}

		public T AsType<T>()
		{
			return (T)(this.RawValue as IConvertible).ToType(typeof(T), null);
		}

		void IValueInfo.CopyJsonTo(System.Text.StringBuilder stringBuilder)
		{
			(this as IValueInfo).CopyJsonTo(new JsonStringBuilderWriter(stringBuilder));
		}

		void IValueInfo.CopyJsonTo(TextWriter writer)
		{
			(this as IValueInfo).CopyJsonTo(new JsonTextWriter(writer));
		}

		void IValueInfo.CopyJsonTo(Stream stream)
		{
			(this as IValueInfo).CopyJsonTo(new JsonStreamWriter(stream));
		}

		void IValueInfo.CopyJsonTo(IJsonWriter writer)
		{
			//writer.Write(this.Token, this.RawValue);
		}

		public bool AsBoolean
		{
			get { return TypeConvert.Convert<object, bool>(this.RawValue, null); }
		}

		public byte AsByte
		{
			get { return TypeConvert.Convert<object, byte>(this.RawValue, null); }
		}

		public short AsInt16
		{
			get { return TypeConvert.Convert<object, Int16>(this.RawValue, null); }
		}

		public int AsInt32
		{
			get { return TypeConvert.Convert<object, Int32>(this.RawValue, null); }
		}

		public long AsInt64
		{
			get { return TypeConvert.Convert<object, Int64>(this.RawValue, null); }
		}

		public sbyte AsSByte
		{
			get { return TypeConvert.Convert<object, SByte>(this.RawValue, null); }
		}

		public ushort AsUInt16
		{
			get { return TypeConvert.Convert<object, UInt16>(this.RawValue, null); }
		}

		public uint AsUInt32
		{
			get { return TypeConvert.Convert<object, UInt32>(this.RawValue, null); }
		}

		public ulong AsUInt64
		{
			get { return TypeConvert.Convert<object, UInt64>(this.RawValue, null); }
		}

		public float AsSingle
		{
			get { return TypeConvert.Convert<object, Single>(this.RawValue, null); }
		}

		public double AsDouble
		{
			get { return TypeConvert.Convert<object, Double>(this.RawValue, null); }
		}

		public decimal AsDecimal
		{
			get { return TypeConvert.Convert<object, Decimal>(this.RawValue, null); }
		}

		public string AsString
		{
			get { return TypeConvert.Convert<object, string>(this.RawValue, null); }
		}

		public DateTime AsDateTime
		{
			get { return TypeConvert.Convert<object, DateTime>(this.RawValue, null); }
		}

		#endregion
	}
	class MethodInvocationInfo : IEquatable<MethodInvocationInfo>
	{
		public MethodBase Method { get; set; }
		public object[] Arguments { get; set; }

		public MethodInvocationInfo(MethodBase method, params object[] arguments)
		{
			this.Method = method;
			this.Arguments = arguments;
		}
		public override int GetHashCode()
		{
			return this.Method.GetHashCode();
		}

		public override bool Equals(object obj)
		{
			return this.Equals(obj as MethodInvocationInfo);
		}

		#region IEquatable<MethodInvocationInfo> Members

		public bool Equals(MethodInvocationInfo other)
		{
			if (other == null)
				return false;

			if (ReferenceEquals(this, other))
				return true;

			if (!this.Method.ToString().Equals(other.Method.ToString()))
				return false;

			if (this.Arguments.Length != other.Arguments.Length)
				return false;

			for (var i = 0; i < this.Arguments.Length; i++)
			{
				if (!Object.Equals(this.Arguments[i], other.Arguments[i]))
					return false;

			}

			return true;
		}

		#endregion

		public override string ToString()
		{
			var result = new System.Text.StringBuilder(100);

			result.Append(this.Method.Name);
			result.Append("(");
			var methodParams = this.Method.GetParameters();
			for (int i = 0; i < methodParams.Length; i++)
			{
				result.Append(methodParams[i].Name);
				result.Append(": ");
				result.Append(this.Arguments[i]);
				if (i != methodParams.Length - 1)
					result.Append(", ");
			}

			result.Append(")");

			return result.ToString();
		}
	}
	class TokenListJsonWriter : IJsonWriter
	{
		public class Token
		{
			public JsonToken JsToken;
			public object JsValue;

			public Token(JsonToken token) : this(token, null) { }
			public Token(JsonToken token, object value)
			{
				this.JsToken = token;
				this.JsValue = value;
			}

			public override string ToString()
			{
				if (JsValue != null)
					return string.Format("{0}:{1}", JsToken, JsValue);
				else
					return JsToken.ToString();
			}
		}

		private List<Token> m_tokens;
		public IList<Token> Tokens { get; private set; }

		public TokenListJsonWriter()
			: this((ISerializationContext)null)
		{

		}
		public TokenListJsonWriter(ISerializationContext context)
		{
			m_tokens = new List<Token>(100);

			this.Tokens = new System.Collections.ObjectModel.ReadOnlyCollection<Token>(m_tokens);
			this.Context = context ?? new AdHocSerializer();
		}

		public TokenListJsonWriter(JsonSerializer_Accessor context)
		{
			m_tokens = new List<Token>(100);

			this.Context = (ISerializationContext)context.CreateContext();
			this.Tokens = new System.Collections.ObjectModel.ReadOnlyCollection<Token>(m_tokens);
		}

		public ISerializationContext Context { get; private set; }
		public long CharactersWritten
		{
			get { return 0; }
		}

		public void Flush()
		{

		}
		public void Reset()
		{
			m_tokens.Clear();
		}

		public void WriteJson(string jsonString)
		{
			m_tokens.Add(new Token(JsonToken.None, jsonString));
		}
		public void WriteJson(char[] jsonString, int start, int len)
		{
			this.WriteJson(new string(jsonString, start, len));
		}

		public void Write(string value)
		{
			m_tokens.Add(new Token(JsonToken.StringLiteral, value));
		}
		public void Write(JsonMember value)
		{
			m_tokens.Add(new Token(JsonToken.Member, value));
		}

		public void Write(int number)
		{
			m_tokens.Add(new Token(JsonToken.Number, number));
		}

		public void Write(uint number)
		{
			m_tokens.Add(new Token(JsonToken.Number, number));
		}

		public void Write(long number)
		{
			m_tokens.Add(new Token(JsonToken.Number, number));
		}

		public void Write(ulong number)
		{
			m_tokens.Add(new Token(JsonToken.Number, number));
		}

		public void Write(float number)
		{
			m_tokens.Add(new Token(JsonToken.Number, number));
		}

		public void Write(double number)
		{
			m_tokens.Add(new Token(JsonToken.Number, number));
		}

		public void Write(decimal number)
		{
			m_tokens.Add(new Token(JsonToken.Number, number));
		}

		public void Write(bool value)
		{
			m_tokens.Add(new Token(JsonToken.Boolean, value));
		}

		public void Write(DateTime datetime)
		{
			m_tokens.Add(new Token(JsonToken.DateTime, datetime));
		}

		public void Write(JsonToken token)
		{
			m_tokens.Add(new Token(token, null));
		}
	}

	class UnseekableStream : Stream
	{
		Stream m_stream;

		public UnseekableStream(Stream baseStream)
		{
			m_stream = baseStream;
		}

		public override bool CanTimeout
		{
			get
			{
				return m_stream.CanTimeout;
			}
		}

		public override int ReadTimeout
		{
			get
			{
				return m_stream.ReadTimeout;
			}
			set
			{
				m_stream.ReadTimeout = value;
			}
		}

		public override int WriteTimeout
		{
			get
			{
				return m_stream.WriteTimeout;
			}
			set
			{
				m_stream.WriteTimeout = value;
			}
		}

		public override bool CanRead
		{
			get { return m_stream.CanRead; }
		}

		public override bool CanSeek
		{
			get { return false; }
		}

		public override bool CanWrite
		{
			get { return m_stream.CanWrite; }
		}

		public override void Flush()
		{
			m_stream.Flush();
		}

		public override long Length
		{
			get { throw new NotSupportedException(); }
		}

		public override long Position
		{
			get
			{
				return m_stream.Position;
			}
			set
			{
				throw new NotSupportedException();
			}
		}

		public override int Read(byte[] buffer, int offset, int count)
		{
			return m_stream.Read(buffer, offset, count);
		}

		public override long Seek(long offset, SeekOrigin origin)
		{
			throw new NotSupportedException();
		}

		public override void SetLength(long value)
		{
			throw new NotSupportedException();
		}

		public override void Write(byte[] buffer, int offset, int count)
		{
			m_stream.Write(buffer, offset, count);
		}

		public override int ReadByte()
		{
			return m_stream.ReadByte();
		}

		public override void WriteByte(byte value)
		{
			m_stream.WriteByte(value);
		}

		public override string ToString()
		{
			return m_stream.ToString();
		}
	}
}
