﻿using Sencha.Serialization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Sencha.Serialization.Contracts;
using System.IO;
using System.Xml;
using Sencha.Serialization.Tests.JsonData;
using System.Runtime.Serialization;
using System.Collections.Generic;

using Token = Sencha.Serialization.Tests.TokenListJsonWriter.Token;
using Sencha.Serialization.Bson;

namespace Sencha.Serialization.Tests
{
	using Test_Data;

	/// <summary>
	///This is a test class for JsonSerializerTest and is intended
	///to contain all JsonSerializerTest Unit Tests
	///</summary>
	[TestClass()]
	public class JsonSerializerTest
	{
		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion

		/// <summary>
		///A test for JsonSerializer Constructor
		///</summary>
		[TestMethod()]
		public void JsonSerializerConstructorTest()
		{
			var serializableType = typeof(MarshalByRefObject);
			var interpreter = MetadataInterpreter.DataContract;
			var descriptor = MetadataDescriber.ReflectionPublicOnly;
			var target = new JsonSerializer(new DataContract(serializableType, interpreter, descriptor));
			var shadow = new JsonSerializer_Accessor(new PrivateObject(target));

			var actual = (TypeDescription)null;
			shadow.m_contract.TryGetKnownType(typeof(MarshalByRefObject), out actual);

			Assert.IsNotNull(actual);
		}

		/// <summary>
		///A test for JsonSerializer Constructor
		///</summary>
		[TestMethod()]
		public void JsonSerializerConstructorTest1()
		{
			var serializableType = typeof(MarshalByRefObject);
			var interpreter = MetadataInterpreter.DataContract;
			var descriptor = MetadataDescriber.ReflectionPublicOnly;
			var contract = new DataContract(serializableType, interpreter, descriptor);
			var target = new JsonSerializer(contract);

			var shadow = new JsonSerializer_Accessor(new PrivateObject(target));

			var actual = (TypeDescription)null;
			shadow.m_contract.TryGetKnownType(typeof(MarshalByRefObject), out actual);

			Assert.IsNotNull(actual);
		}

		/// <summary>
		///A test for JsonSerializer Constructor
		///</summary>
		[TestMethod()]
		public void JsonSerializerConstructorTest2()
		{
			var serializableType = typeof(MarshalByRefObject);
			var knownTypes = new Type[] { typeof(IAsyncResult) };
			var target = new JsonSerializer(serializableType, knownTypes);
			var shadow = new JsonSerializer_Accessor(new PrivateObject(target));

			var actual = (TypeDescription)null;
			shadow.m_contract.TryGetKnownType(typeof(IAsyncResult), out actual);

			Assert.IsNotNull(actual);
		}

		// read/write to seeakable stream
		[TestMethod()]
		public void ReadWriteToSeekableStream()
		{
			var stream = new MemoryStream();
			var writer = new JsonStreamWriter(stream);
			var reader = new JsonStreamReader(stream);
			var value = new ReadWriteTestObject();

			writer.WriteValue(value);
			writer.Flush();
			stream.Position = 0;

			var actual = reader.ReadValue<ReadWriteTestObject>(false);

			Assert.AreEqual(value, actual);
		}

		// read/write to unseekable stream
		[TestMethod()]
		public void ReadWriteToUnseekableStream()
		{
			var stream = new MemoryStream();
			var ustream = new UnseekableStream(stream);
			var writer = new JsonStreamWriter(ustream);
			var reader = new JsonStreamReader(ustream);
			var value = new ReadWriteTestObject();

			writer.WriteValue(value);
			writer.Flush();
			stream.Position = 0;

			var actual = reader.ReadValue<ReadWriteTestObject>(false);

			Assert.AreEqual(value, actual);
		}

		// read/write to string/string builder/text
		[TestMethod()]
		public void ReadWriteToStringStream()
		{
			var sb = new System.Text.StringBuilder();
			var writer = new JsonStringBuilderWriter(sb);

			var value = new ReadWriteTestObject();

			writer.WriteValue(value);
			writer.Flush();

			var reader = new JsonStringReader(sb.ToString());
			var actual = reader.ReadValue<ReadWriteTestObject>(false);

			Assert.AreEqual(value, actual);
		}
		[TestMethod()]
		public void ReadWriteToStringBuilderStream()
		{
			var sb = new System.Text.StringBuilder();
			var writer = new JsonStringBuilderWriter(sb);
			var reader = new JsonStringBuilderReader(sb);
			var value = new ReadWriteTestObject();

			writer.WriteValue(value);
			writer.Flush();

			var actual = reader.ReadValue<ReadWriteTestObject>(false);

			Assert.AreEqual(value, actual);
		}
		[TestMethod()]
		public void ReadWriteToTextReaderWriterStream()
		{
			var sb = new System.Text.StringBuilder();
			var writer = new JsonTextWriter(new StringWriter(sb));

			var value = new ReadWriteTestObject();

			writer.WriteValue(value);
			writer.Flush();

			var reader = new JsonTextReader(new StringReader(sb.ToString()));
			var actual = reader.ReadValue<ReadWriteTestObject>(false);

			Assert.AreEqual(value, actual);
		}

		// features
		/// <summary>
		///A test for StringInterchange
		///</summary>
		[TestMethod()]
		public void DictionaryObjectInterchangeTest()
		{
			var target = new JsonSerializer(typeof(IDictionary<string, int>));
			var reader = new StringReader("{a:1,b:2,c:3}");

			var actual = target.ReadObject<IDictionary<string, int>>(reader);

			Assert.IsNotNull(actual);
			Assert.AreEqual(3, actual.Count);
			Assert.AreEqual(1, actual["a"]);
			Assert.AreEqual(2, actual["b"]);
			Assert.AreEqual(3, actual["c"]);
		}

		/// <summary>
		///A test for StringInterchange
		///</summary>
		[TestMethod()]
		public void StringInterchangeTest()
		{
			var target = new JsonSerializer(typeof(string)) { StringInterchange = true };
			var reader = new StringReader("[1,2,3,{'4':5}, [6,7,8,'9','10','11,12']]");

			var actual = target.ReadObject<string>(reader);
			Assert.IsNotNull(actual);
		}

		/// <summary>
		///A test for StringInterchange
		///</summary>
		[TestMethod()]
		public void StringInterchangeNegativeTest()
		{
			var target = new JsonSerializer(typeof(string)) { StringInterchange = false };
			var reader = new StringReader("[1,2,3,{'4':5}, [6,7,8,'9','10','11,12']]");

			try
			{
				var actual = target.ReadObject<string>(reader);
				Assert.Fail();
			}
			catch (SerializationException)
			{

			}
		}

		/// <summary>
		///A test for SkipUnknownMember
		///</summary>
		[TestMethod()]
		public void SkipUnknownMemberTest()
		{
			var target = new JsonSerializer(typeof(KeyValue<string, int>)) { SkipUnknownMember = true };
			var reader = new StringReader("{ Key: \"a\", Value: 1, UnknownMember: 3 }");

			var actual = target.ReadObject<KeyValue<string, int>>(reader);
			Assert.IsNotNull(actual);
			Assert.AreEqual("a", actual.Key);
			Assert.AreEqual(1, actual.Value);
		}

		/// <summary>
		///A test for SkipUnknownMember
		///</summary>
		[TestMethod()]
		public void SkipUnknownMemberNegativeTest()
		{
			var target = new JsonSerializer(typeof(KeyValue<string, int>)) { SkipUnknownMember = false };
			var reader = new StringReader("{ Key: \"a\", Value: 1, UnknownMember: 3 }");

			try
			{
				var actual = target.ReadObject<KeyValue<string, int>>(reader);
				Assert.Fail();
			}
			catch (SerializationException)
			{

			}
		}

		/// <summary>
		///A test for SkipUnaccessibleMembers
		///</summary>
		[TestMethod()]
		public void SkipUnaccessibleMembersTest()
		{
			var target = new JsonSerializer(typeof(KeyValuePair<string, int>)) { SkipUnaccessibleMembers = true };
			var reader = new StringReader("{ Key: 'a', Value: 1 }");

			var actual = target.ReadObject<KeyValuePair<string, int>>(reader);
			Assert.IsNotNull(actual);
			Assert.AreEqual(default(string), actual.Key);
			Assert.AreEqual(default(int), actual.Value);
		}


		/// <summary>
		///A test for SkipUnaccessibleMembers
		///</summary>
		[TestMethod()]
		public void SkipUnaccessibleMembersNegativeTest()
		{
			var target = new JsonSerializer(typeof(KeyValuePair<string, int>)) { SkipUnaccessibleMembers = false };
			var reader = new StringReader("{ Key: 'a', Value: 1 }");

			try
			{
				var actual = target.ReadObject<KeyValuePair<string, int>>(reader);
				Assert.Fail();
			}
			catch (SerializationException)
			{

			}
		}

		/// <summary>
		///A test for SerializeByteArrayAsBase64
		///</summary>
		[TestMethod()]
		public void SerializeByteArrayAsBase64Test()
		{
			var target = new JsonSerializer(typeof(byte[])) { SerializeByteArrayAsBase64 = true };
			var writer = new StringWriter();

			var value = BitConverter.GetBytes(124566);
			var expected = "\"" + Convert.ToBase64String(value) + "\"";

			target.WriteObject(writer, value);

			var actual = writer.ToString();

			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for SerializeByteArrayAsBase64
		///</summary>
		[TestMethod()]
		public void SerializeByteArrayAsBase64DisabledTest()
		{
			var target = new JsonSerializer(typeof(byte[])) { SerializeByteArrayAsBase64 = false };
			var writer = new TokenListJsonWriter(target.CreateContext());

			var value = BitConverter.GetBytes(124566);
			var expected = new[] { 
				new Token(JsonToken.BeginArray), 
					new Token(JsonToken.Number, (int)value[0]),  // minimal number width(in bits) passed to writer is 32, so event bytes written as Int32
					new Token(JsonToken.Number, (int)value[1]), 
					new Token(JsonToken.Number, (int)value[2]), 
					new Token(JsonToken.Number, (int)value[3]), 
				new Token(JsonToken.EndOfArray) 
			};

			target.WriteObject(writer, value);

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}

		/// <summary>
		///A test for ArrayInterchange
		///</summary>
		[TestMethod()]
		public void ArrayInterchangeTest()
		{
			var target = new JsonSerializer(typeof(int[])) { ArrayInterchange = true };
			var reader = new StringReader("42");

			var actual = target.ReadObject<int[]>(reader);
			Assert.IsNotNull(actual);
			Assert.AreEqual(1, actual.Length);
			Assert.AreEqual(42, actual[0]);
		}

		/// <summary>
		///A test for MaxItemsInObjectGraph
		///</summary>
		[TestMethod()]
		public void MaxItemsInObjectGraphReadTest()
		{
			var target = new JsonSerializer(typeof(object)) { MaxItemsInObjectGraph = 3 };
			var reader = new StringReader("[1,2,3,{'4':5}, [6,7,8,'9','10','11,12']]");

			try
			{
				target.ReadObject<object>(reader);
				Assert.Fail("Should fail");
			}
			catch (Exceptions.SerializationGraphIsTooBig e) { }
		}

		/// <summary>
		///A test for MaxItemsInObjectGraph
		///</summary>
		[TestMethod()]
		public void MaxItemsInObjectGraphWriteTest()
		{
			var target = new JsonSerializer(typeof(object)) { MaxItemsInObjectGraph = 3 };
			var value = new object[] { null }; value[0] = value; // recursion
			var writer = new StringWriter();

			try
			{
				target.WriteObject(writer, value);
				Assert.Fail("Should fail");
			}
			catch (Exceptions.SerializationGraphIsTooBig e) { }
		}
	}
}
