﻿using Sencha.Serialization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for JsonReaderBaseTest and is intended
	///to contain all JsonReaderBaseTest Unit Tests
	///</summary>
	[TestClass()]
	public class JsonReaderBaseTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		internal virtual JsonReaderBase_Accessor CreateJsonBufferedReader(string json)
		{
			var instance = new PrivateObject(new JsonStringReader(json, 10), new PrivateType(typeof(JsonReaderBase)));
			var target = new JsonReaderBase_Accessor(instance);
			return target;
		}


		/// <summary>
		///A test for IsLiteralTerminator
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsLiteralTerminatorNonQuotedTest()
		{
			var json = "[]";
			var target = CreateJsonBufferedReader(json);

			var nonQuotedTerminators = "\t\r\n []{}:,";
			for (int i = 0; i < nonQuotedTerminators.Length; i++)
			{
				char ch = nonQuotedTerminators[i];
				bool quoted = false;
				char quoteCh = '"';
				bool escaped = false;
				bool expected = true;
				bool actual = target.IsLiteralTerminator(ch, quoted, quoteCh, escaped, false);
				Assert.AreEqual(expected, actual, string.Format("Symbol ({0}) should be considered as unquoted string terminator", ch));
			}
		}

		/// <summary>
		///A test for IsLiteralTerminator
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsLiteralTerminatorNonQuotedEosTest()
		{
			var json = "[]";
			var target = CreateJsonBufferedReader(json);

			char ch = 'a';
			bool quoted = false;
			char quoteCh = '"';
			bool escaped = false;
			bool expected = true;
			var eos = true;
			bool actual = target.IsLiteralTerminator(ch, quoted, quoteCh, escaped, eos);
			Assert.AreEqual(expected, actual, string.Format("End of stream should be considered as unquoted string terminator"));

		}

		/// <summary>
		///A test for IsLiteralTerminator
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsLiteralTerminatorQuotedTest()
		{
			var json = "[]";
			var target = CreateJsonBufferedReader(json);

			// quote
			char ch = '"';
			bool quoted = true;
			char quoteCh = '"';
			bool escaped = false;
			bool expected = true;
			bool actual = target.IsLiteralTerminator(ch, quoted, quoteCh, escaped, false);
			Assert.AreEqual(expected, actual);

			// escaped quote
			ch = '"';
			quoted = true;
			quoteCh = '"';
			escaped = true;
			expected = false;
			actual = target.IsLiteralTerminator(ch, quoted, quoteCh, escaped, false);
			Assert.AreEqual(expected, actual);

			// single quote
			ch = '\'';
			quoted = true;
			quoteCh = '"';
			escaped = false;
			expected = false;
			actual = target.IsLiteralTerminator(ch, quoted, quoteCh, escaped, false);
			Assert.AreEqual(expected, actual);

			// other char
			ch = '\t';
			quoted = true;
			quoteCh = '"';
			escaped = false;
			expected = false;
			actual = target.IsLiteralTerminator(ch, quoted, quoteCh, escaped, false);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for IsNumber
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsNumberTest()
		{
			var json = "1000024.1";
			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer;

			int start = 0;
			int len = json.Length;
			bool expected = true;
			bool actual;
			actual = target.IsNumber(buffer, start, len);
			Assert.AreEqual(expected, actual, string.Format("'{0}' should be considered as number", json));
		}

		/// <summary>
		///A test for IsNumber
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsNumberMinusTest()
		{
			var json = "-100002";
			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer;

			int start = 0;
			int len = json.Length;
			bool expected = true;
			bool actual;
			actual = target.IsNumber(buffer, start, len);
			Assert.AreEqual(expected, actual, string.Format("'{0}' should be considered as number", json));
		}

		/// <summary>
		///A test for IsNumber
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsNumberSciTest()
		{
			var json = "1000024e-5";
			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer; ;

			int start = 0;
			int len = json.Length;
			bool expected = true;
			bool actual;
			actual = target.IsNumber(buffer, start, len);
			Assert.AreEqual(expected, actual, string.Format("'{0}' should be considered as number", json));
		}

		/// <summary>
		///A test for IsNumber
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsNumberSci2Test()
		{
			var json = "1000024E10";
			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer;

			int start = 0;
			int len = json.Length;
			bool expected = true;
			bool actual;
			actual = target.IsNumber(buffer, start, len);
			Assert.AreEqual(expected, actual, string.Format("'{0}' should be considered as number", json));
		}

		/// <summary>
		///A test for IsNumber
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsNumberBadTest()
		{
			var json = "1000024e1e0";
			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer;

			int start = 0;
			int len = json.Length;
			bool expected = false;
			bool actual;
			actual = target.IsNumber(buffer, start, len);
			Assert.AreEqual(expected, actual, string.Format("'{0}' should be considered as number", json));
		}

		/// <summary>
		///A test for IsNumber
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsNumberBad2Test()
		{
			var json = "1000.02.40";
			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer;

			int start = 0;
			int len = json.Length;
			bool expected = false;
			bool actual;
			actual = target.IsNumber(buffer, start, len);
			Assert.AreEqual(expected, actual, string.Format("'{0}' should not to be considered as number", json));
		}

		/// <summary>
		///A test for IsNumber
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void IsNumberBad3Test()
		{
			var json = "1000,40";
			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer;

			int start = 0;
			int len = json.Length;
			bool expected = false;
			bool actual;
			actual = target.IsNumber(buffer, start, len);
			Assert.AreEqual(expected, actual, string.Format("'{0}' should not to be considered as number", json));
		}

		/// <summary>
		///A test for LookupAtSkipWhitespace
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void LookupAtSkipWhitespaceTest()
		{
			var json = "    \t\t\r\n:   ";
			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer;

			int start = 0;
			int len = 1;
			string matchString = ":";
			bool expected = true;
			bool actual = target.LookupAtSkipWhitespace(buffer, start, len, matchString);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for NextLexeme
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void NextLexemeTest()
		{
			var json = "[\"\",a,b,c,\r\n\"a \\'b\",	1234.1,{ \"a\"   :\"'abirvalg\", \"ba\": \r\n\rtrue}]";
			var expected = new string[] { "[", "", "a", "b", "c", "a \\'b", "1234.1", "{", "a", "'abirvalg", "ba", "true", "}", "]" };
			var target = CreateJsonBufferedReader(json);

			for (int i = 0; i < expected.Length; i++)
			{
				int start = 0;
				int len = 0;
				bool quoted = false;
				bool isMember = false;
				var buffer = target.NextLexeme(ref start, ref len, ref quoted, ref isMember);

				if (buffer == null)
					Assert.Fail("Unexpected end of stream");

				var lexeme = new string(buffer.GetChars(), buffer.Offset + start, len);

				buffer.Fixate(start + len + (quoted ? 1 : 0));

				Assert.AreEqual(expected[i], lexeme);
			}
		}

		/// <summary>
		///A test for NextToken
		///</summary>
		[TestMethod()]
		public void NextTokenTest()
		{
			var json = "[\"\",a,b,c,\r\n\"a \\'b\",	1234.1,{ \"a\"   :\"'abirvalg\", \"ba\": \r\n\rtrue}]";
			var expected = new JsonToken[] { JsonToken.BeginArray, JsonToken.StringLiteral, JsonToken.StringLiteral, JsonToken.StringLiteral, JsonToken.StringLiteral, JsonToken.StringLiteral, 
				JsonToken.Number, JsonToken.BeginObject, JsonToken.Member, JsonToken.StringLiteral, JsonToken.Member, JsonToken.Boolean, JsonToken.EndOfObject, JsonToken.EndOfArray };
			var target = CreateJsonBufferedReader(json);

			for (int i = 0; i < expected.Length; i++)
			{
				var actual = target.NextToken();

				if (actual == false)
					Assert.Fail("Unexpected end of stream");

				var token = target.Token;

				Assert.AreEqual(expected[i], token);
			}
		}

		/// <summary>
		///A test for Unencode
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void UnescapeTest()
		{
			var json = "\"abcde\\n\\r\\b\\f\\t\\\"\\'\\u0430\\x40\\064\"";
			var expected = "abcde\n\r\b\f\t\"'а@@";

			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer;
			buffer.ReadNextBlock();

			int start = 1;
			int len = json.Length - 2;
			string actual = target.Unescape(buffer, start, len);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Unencode
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void LineNumbersTest()
		{
			var json = "{\r\n \"a\":\r\n 123\r\n, \"b\":\r\n [true, false, true, false]\r\n}\r\n";
			var lineNumbers = new[] { 1, 2, 3, 4, 5, 5, 5,  5,  5,  5, 6 };
			var columns = new[]     { 1, 3, 3, 4, 3, 4, 10, 17, 23, 28, 2 };

			var target = CreateJsonBufferedReader(json);
			var buffer = target.m_buffer;

			for (var i = 0; i < lineNumbers.Length; i++)
			{
				target.NextToken();
				var lineNum = lineNumbers[i];
				var colNum = columns[i];

				Assert.AreEqual(lineNum, buffer.LineNumber);
				Assert.AreEqual(colNum, buffer.ColumnNumber);
			}
		}
	}
}
