﻿using Sencha.Serialization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for JsonStreamReaderTest and is intended
	///to contain all JsonStreamReaderTest Unit Tests
	///</summary>
	[TestClass()]
	public class JsonStreamReaderTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for JsonStreamReader Constructor
		///</summary>
		[TestMethod()]
		public void JsonStreamReaderConstructorTest()
		{
			Stream stream = new MemoryStream();
			ISerializationContext context = new AdHocSerializer();
			int bufferSize = 1024;
			JsonStreamReader target = new JsonStreamReader(stream, context, bufferSize);

		}

		/// <summary>
		///A test for JsonStreamReader Constructor
		///</summary>
		[TestMethod()]
		public void JsonStreamReaderConstructorWithUnseekableStreamTest()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			ISerializationContext context = new AdHocSerializer();
			int bufferSize = 1024;
			JsonStreamReader target = new JsonStreamReader(stream, context, bufferSize);

		}

		/// <summary>
		///A test for JsonStreamReader Constructor
		///</summary>
		[TestMethod()]
		public void JsonStreamReaderConstructorNoContextTest()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			int bufferSize = 1024;
			JsonStreamReader target = new JsonStreamReader(stream, bufferSize);

		}

		/// <summary>
		///A test for JsonStreamReader Constructor
		///</summary>
		[TestMethod()]
		public void JsonStreamReaderConstructorTest2()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamReader target = new JsonStreamReader(stream);
		}

		/// <summary>
		///A test for FillBuffer
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void FillBufferTest()
		{
			var testString = "my test string";
			var bytes = System.Text.Encoding.UTF8.GetBytes(testString);
			Stream stream = new UnseekableStream(new MemoryStream(bytes));
			JsonStreamReader reader = new JsonStreamReader(stream);
			JsonStreamReader_Accessor target = new JsonStreamReader_Accessor(new PrivateObject(reader));
			char[] buffer = new char[1024];
			int index = 52;
			int expected = index + testString.Length;
			char[] expectedChars = testString.ToCharArray();
			int actual;
			actual = target.FillBuffer(buffer, index);
			Assert.AreEqual(expected, actual);

			for (var i = index; i < expectedChars.Length; i++)
				Assert.AreEqual(expectedChars[i - index], buffer[i], "wrong character copied in buffer");
		}

		/// <summary>
		///A test for JsonStreamReader Constructor
		///</summary>
		[TestMethod()]
		public void ToStringTest()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamReader target = new JsonStreamReader(stream);
			Assert.IsNotNull(target.ToString());
		}
	}
}
