﻿using Sencha.Serialization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for JsonStreamWriterTest and is intended
	///to contain all JsonStreamWriterTest Unit Tests
	///</summary>
	[TestClass()]
	public class JsonStreamWriterTest
	{


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for JsonStreamWriter Constructor
		///</summary>
		[TestMethod()]
		public void JsonStreamWriterConstructorTest()
		{
			Stream stream = new MemoryStream();
			ISerializationContext context = new AdHocSerializer();
			JsonStreamWriter target = new JsonStreamWriter(stream, context);
		}

		/// <summary>
		///A test for JsonStreamWriter Constructor
		///</summary>
		[TestMethod()]
		public void JsonStreamWriterConstructorNonSeekableStreamTest()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			ISerializationContext context = new AdHocSerializer();
			JsonStreamWriter target = new JsonStreamWriter(stream, context);
		}

		/// <summary>
		///A test for JsonStreamWriter Constructor
		///</summary>
		[TestMethod()]
		public void JsonStreamWriterConstructorTest1()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamWriter target = new JsonStreamWriter(stream);
		}

		/// <summary>
		///A test for Flush
		///</summary>
		[TestMethod()]
		public void FlushTest()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamWriter target = new JsonStreamWriter(stream);

			target.WriteJson("myJson");

			var position = stream.Position;

			Assert.AreEqual(0, position);

			target.Flush();

			position = stream.Position;

			Assert.AreNotEqual(0, position);
		}

		/// <summary>
		///A test for ToString
		///</summary>
		[TestMethod()]
		public void ToStringTest()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamWriter target = new JsonStreamWriter(stream);
			Assert.IsNotNull(target.ToString());
		}

		/// <summary>
		///A test for WriteJson
		///</summary>
		[TestMethod()]
		public void WriteJsonTest()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamWriter target = new JsonStreamWriter(stream);
			string jsonString = "My arbitary json";
			target.WriteJson(jsonString);
			target.Flush();

			var position = stream.Position;

			Assert.AreNotEqual(0, position);
		}

		/// <summary>
		///A test for WriteJson
		///</summary>
		[TestMethod()]
		public void WriteEmptyJsonTest()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamWriter target = new JsonStreamWriter(stream);
			string jsonString = String.Empty;
			target.WriteJson(jsonString);
			target.Flush();

			var position = stream.Position;

			Assert.AreEqual(0, position);
		}

		/// <summary>
		///A test for WriteJson
		///</summary>
		[TestMethod()]
		public void WriteJsonTestWithCharArray()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamWriter target = new JsonStreamWriter(stream);
			char[] jsonString = "My arbitary json".ToCharArray();
			int len = 7;
			int index = 1;
			target.WriteJson(jsonString, index, len);
			target.Flush();

			var position = stream.Position - 1;

			Assert.AreEqual(len - index, position);
		}

		/// <summary>
		///A test for WriteJson
		///</summary>
		[TestMethod()]
		public void WriteEmptyJsonTestWithCharArray()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamWriter target = new JsonStreamWriter(stream);
			char[] jsonString = "My arbitary json".ToCharArray();
			int len = 0;
			int index = 1;
			target.WriteJson(jsonString, index, len);
			target.Flush();

			var position = stream.Position;

			Assert.AreEqual(len, position);
		}

		/// <summary>
		///A test for Stream
		///</summary>
		[TestMethod()]
		public void StreamTest()
		{
			Stream stream = new UnseekableStream(new MemoryStream());
			JsonStreamWriter target = new JsonStreamWriter(stream);
			Stream actual;
			actual = target.Stream;

			Assert.AreSame(stream, actual);
		}
	}
}
