﻿using Sencha.Serialization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace Sencha.Serialization.Tests
{
    
    
    /// <summary>
    ///This is a test class for JsonWriterBaseTest and is intended
    ///to contain all JsonWriterBaseTest Unit Tests
    ///</summary>
	[TestClass()]
	public class JsonWriterBaseTest
	{
		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		internal virtual JsonWriterBase_Accessor CreateJsonWriterBase_Accessor()
		{
			var instance = new JsonStringBuilderWriter(new System.Text.StringBuilder());
			var target = new JsonWriterBase_Accessor(new PrivateObject(instance, new PrivateType(typeof(JsonWriterBase))));
			return target;
		}

		private JsonStreamWriter CreateJsonWriterBase()
		{
			var stream = new System.IO.MemoryStream();
			return new JsonStreamWriter(stream);
		}

		/// <summary>
		///A test for Flush
		///</summary>
		[TestMethod()]
		public void FlushTest()
		{
			var target = CreateJsonWriterBase();
			target.Write(JsonToken.Null);

			var bytesInStream = target.Stream.Position;
			Assert.AreEqual(bytesInStream, 0);

			target.Flush();

			bytesInStream = target.Stream.Position;
			Assert.AreNotSame(bytesInStream, 0);
		}

		/// <summary>
		///A test for Reset
		///</summary>
		[TestMethod()]
		public void ResetTest()
		{
			var target = CreateJsonWriterBase();
			target.Write(JsonToken.Null);

			var charsInStream = target.CharactersWritten;
			Assert.AreNotEqual(charsInStream, 0);

			target.Reset();

			charsInStream = target.CharactersWritten;
			Assert.AreEqual(charsInStream, 0);
		}

		/// <summary>
		///A test for Write
		///</summary>
		[TestMethod()]
		public void WriteTest()
		{
			var target = CreateJsonWriterBase();
			
			target.Write(JsonToken.Null);
			target.Write(JsonToken.BeginArray);
			target.Write(JsonToken.EndOfArray);
			target.Write(JsonToken.BeginObject);
			target.Write(JsonToken.EndOfObject);
			target.Write(true);
			target.Write(false);
			target.Write(DateTime.UtcNow);			
			target.Write((JsonMember)"memberName");
			target.Write((byte)1);
			target.Write((sbyte)-1);
			target.Write((short)-11234);
			target.Write((ushort)63234);
			target.Write((int)12363234);
			target.Write((uint)412363234);
			target.Write((long)-123212363234);
			target.Write((ulong)123212363234);
			target.Write((float)1232123234.1);
			target.Write((double)1232123234.123414);
			target.Write((decimal)1232123234.123414);
			target.Write("qweqwe\u0012\r\n\a\b\'\"\"\"\a\t\vawqwe123459абырвалг�");

			target.Flush();

			var bytesInStream = target.Stream.Position;
			Assert.AreNotSame(bytesInStream, 0);
		}
	}
}
