﻿
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Sencha.Serialization.Contracts;
using Sencha.Serialization.Serializers;
using Sencha.Serialization;

using Token = Sencha.Serialization.Tests.TokenListJsonWriter.Token;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for ArraySerializationStrategyTest and is intended
	///to contain all ArraySerializationStrategyTest Unit Tests
	///</summary>
	[TestClass()]
	public class ArraySerializerTest
	{
		public class TestObject
		{
			public int IntField;
			public string StringProperty { get; set; }
		}
		public class MyStringList : IEnumerable<string>
		{
			private List<string> m_list;

			public MyStringList()
			{
				m_list = new List<string>();
			}
			public MyStringList(string[] strs)
			{
				m_list = new List<string>(strs);
			}

			public void Add(string value)
			{
				m_list.Add(value);
			}

			public int Count { get { return m_list.Count; } }

			public string this[int index] { get { return m_list[index]; } }

			#region IEnumerable<string> Members

			public IEnumerator<string> GetEnumerator()
			{
				return m_list.GetEnumerator();
			}

			#endregion

			#region IEnumerable Members

			IEnumerator IEnumerable.GetEnumerator()
			{
				return m_list.GetEnumerator();
			}

			#endregion
		}

		private DataContract IntArrayContract = new DataContract(typeof(int[]), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private DataContract IntListContract = new DataContract(typeof(List<int>), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private string IntArrayJson = "[10, 5, 4]";
		private string IntSingleJson = "10";

		private DataContract ObjectArrayContract = new DataContract(typeof(TestObject[]), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private DataContract ObjectListContract = new DataContract(typeof(List<TestObject>), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private string ObjectArrayContractJson = "[{IntField:1,StringProperty:\"a\"}, {IntField:2,StringProperty:\"fh\"}, {IntField:3,StringProperty:\"def\"}, {IntField:4,StringProperty:\"cdef\"}]";

		private DataContract StringArrayContract = new DataContract(typeof(string[]), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private DataContract StringListContract = new DataContract(typeof(List<string>), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private DataContract MyStringListContract = new DataContract(typeof(MyStringList), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);

		private string StringArrayJson = "[\"1\",\"2\",\"3\"]";


		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}


		/// <summary>
		///A test for ArraySerializationStrategy Constructor
		///</summary>
		[TestMethod()]
		public void ArraySerializationStrategyConstructorTest()
		{
			var enumerableType = IntArrayContract.Root;
			var target = new ArraySerializer<int[]>(enumerableType);
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeIntArrayTest()
		{
			var enumerableType = IntArrayContract.Root;
			var target = new ArraySerializer<int[]>(enumerableType);
			var reader = new JsonStringReader(IntArrayJson);
			
			object actual;
			actual = target.Deserialize(reader);
			Assert.IsInstanceOfType(actual, IntArrayContract.Root.GetClrType());
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeIntArrayWithSingleValueTest()
		{
			var contract = IntArrayContract;
			var enumerableType = IntArrayContract.Root;
			var target = new ArraySerializer<int[]>(enumerableType);
			var reader = new JsonStringReader(IntSingleJson, new AdHocSerializer { Features = Features.ArrayInterchange });

			object actual;
			actual = target.Deserialize(reader);
			Assert.IsInstanceOfType(actual, IntArrayContract.Root.GetClrType());
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeIntArrayWithSingleValueNegativeTest()
		{
			var contract = IntArrayContract;
			var enumerableType = IntArrayContract.Root;
			var target = new ArraySerializer<int[]>(enumerableType);			
			var reader = new JsonStringReader(IntSingleJson, new AdHocSerializer { Features = 0 });

			try
			{
				object actual;
				actual = target.Deserialize(reader);
				Assert.Fail("Should fail");
			}
			catch (System.Runtime.Serialization.SerializationException)
			{

			}
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeIntListTest()
		{
			var contract = IntListContract;
			var enumerableType = IntListContract.Root;
			var target = new ArraySerializer<List<int>>(enumerableType);			
			var reader = new JsonStringReader(IntArrayJson);

			object actual;
			actual = target.Deserialize(reader);
			Assert.IsInstanceOfType(actual, IntListContract.Root.GetClrType());
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeStringArrayTest()
		{
			var contract = StringArrayContract;
			var enumerableType = StringArrayContract.Root;
			var target = new ArraySerializer<string[]>(enumerableType);			
			var reader = new JsonStringReader(StringArrayJson);


			object actual;
			actual = target.Deserialize(reader);
			Assert.IsInstanceOfType(actual, StringArrayContract.Root.GetClrType());
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeStringListTest()
		{
			var contract = StringListContract;
			var enumerableType = StringListContract.Root;
			var target = new ArraySerializer<List<string>>(enumerableType);
			var reader = new JsonStringReader(StringArrayJson);

			object actual;
			actual = target.Deserialize(reader);
			Assert.IsInstanceOfType(actual, StringListContract.Root.GetClrType());
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeObjectArrayTest()
		{
			var contract = ObjectArrayContract;
			var enumerableType = ObjectArrayContract.Root;
			var target = new ArraySerializer<TestObject[]>(enumerableType);
			var reader = new JsonStringReader(ObjectArrayContractJson);

			object actual;
			actual = target.Deserialize(reader);
			Assert.IsInstanceOfType(actual, ObjectArrayContract.Root.GetClrType());
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeObjectListTest()
		{
			var contract = ObjectListContract;
			var enumerableType = ObjectListContract.Root;
			var target = new ArraySerializer<List<TestObject>>(enumerableType);
			var reader = new JsonStringReader(ObjectArrayContractJson);

			object actual;
			actual = target.Deserialize(reader);
			Assert.IsInstanceOfType(actual, ObjectListContract.Root.GetClrType());
		}

		/// <summary>
		///A test for GetElementType
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void GetElementTypeTest()
		{
			var target = new ArraySerializer_Accessor<int[]>(IntArrayContract.Root);
			var type = IntArrayContract.Root.GetClrType();
			Type expected = typeof(int);
			Type actual;
			actual = target.GetElementType(type);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeTest()
		{
			var contract = IntArrayContract;
			var enumerableType = IntArrayContract.Root;
			var target = new ArraySerializer<int[]>(enumerableType);			
			var writer = new TokenListJsonWriter();


			var value = new int[] { 0, 2, 3 };
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginArray), 
					new Token(JsonToken.Number, 0), 
					new Token(JsonToken.Number, 2), 
					new Token(JsonToken.Number, 3), 
				new Token(JsonToken.EndOfArray) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}
	}
}
