﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

using Sencha.Serialization;
using Sencha.Serialization.Contracts;
using Sencha.Serialization.Serializers;

using Token = Sencha.Serialization.Tests.TokenListJsonWriter.Token;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for ByteArraySerializationStrategyTest and is intended
	///to contain all ByteArraySerializationStrategyTest Unit Tests
	///</summary>
	[TestClass()]
	public class Base64SerializerTest
	{

		public DataContract ByteArrayContract = new DataContract(typeof(Byte[]), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}


		/// <summary>
		///A test for ByteArraySerializationStrategy Constructor
		///</summary>
		[TestMethod()]
		public void ByteArraySerializationStrategyConstructorTest()
		{
			TypeDescription byteArrayType = ByteArrayContract.Root; 
			var target = new Base64Serializer(byteArrayType);
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeTest()
		{
			var contract = ByteArrayContract;
			var bytes = BitConverter.GetBytes(ulong.MaxValue);
			var base64String = "\"" + Convert.ToBase64String(bytes) + "\"";
			var byteArrayType = contract.Root;
			var target = new Base64Serializer(byteArrayType);			
			var reader = new JsonStringReader(base64String);
			
			var expected = bytes;
			var actual = target.Deserialize(reader) as byte[];

			Assert.IsNotNull(actual);
			Assert.AreEqual(expected.Length, actual.Length, "arrays has different length");
			for (var i = 0; i < actual.Length; i++)
				Assert.AreEqual(expected[i], actual[i], "arrays not same");
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeTest()
		{
			var contract = ByteArrayContract;
			var bytes = BitConverter.GetBytes(ulong.MaxValue);
			var base64String = Convert.ToBase64String(bytes);
			var byteArrayType = contract.Root;
			var target = new Base64Serializer(byteArrayType);			
			var writer = new TokenListJsonWriter();

			target.Serialize(writer, bytes);

			var expected = new Token[] { new Token(JsonToken.StringLiteral, base64String) };

			Assert.IsNotNull(writer.Tokens);
			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}
	}
}
