﻿using Sencha.Serialization.Serializers;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Sencha.Serialization.Contracts;
using Sencha.Serialization;
using System.ComponentModel;

using Token = Sencha.Serialization.Tests.TokenListJsonWriter.Token;

namespace Sencha.Serialization.Tests
{
    
    
    /// <summary>
    ///This is a test class for ConvertibleSerializationStrategyTest and is intended
    ///to contain all ConvertibleSerializationStrategyTest Unit Tests
    ///</summary>
	[TestClass()]
	public class ConvertibleSerializerTest
	{
		[TypeConverter(typeof(MyConvertibleTypeConverter))]
		public class MyConvertibleType
		{
			public string Value;
		}
		public class MyConvertibleTypeConverter : TypeConverter
		{
			public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
			{
				if (sourceType == typeof(string))
					return true;

				return base.CanConvertFrom(context, sourceType);
			}
			public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
			{
				if (destinationType == typeof(string))
					return true;

				return base.CanConvertTo(context, destinationType);
			}
			public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
			{
				if (value is String || value == null)
					return new MyConvertibleType() { Value = value as string };

				return base.ConvertFrom(context, culture, value);
			}
			public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
			{
				if (destinationType == typeof(string))
					return value != null ? (value as MyConvertibleType).Value : null;

				return base.ConvertTo(context, culture, value, destinationType);
			}
		}

		[TypeConverter(typeof(MyNonConvertibleToStringTypeConverter))]
		public class MyNonConvertibleToStringType
		{
			public string Value;
		}
		public class MyNonConvertibleToStringTypeConverter : TypeConverter
		{
		}


		public DataContract ConvertibleContract = new DataContract(typeof(MyConvertibleType), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract NonConvertibleContract = new DataContract(typeof(MyNonConvertibleToStringType), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);

		public const string ConvertibleText = "This is text for convertible object";
		public string ConvertibleJson = "\"" + ConvertibleText + "\"";

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}


		/// <summary>
		///A test for ConvertibleSerializationStrategy Constructor
		///</summary>
		[TestMethod()]
		public void ConvertibleSerializationStrategyConstructorTest()
		{
			TypeDescription convertibleType = ConvertibleContract.Root;
			var target = new ConvertibleSerializer<MyConvertibleType>(convertibleType);
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		public void DeserializeTest()
		{
			var contract = ConvertibleContract;
			var convertibleType = contract.Root;
			var target = new ConvertibleSerializer<MyConvertibleType>(convertibleType);

			var reader = new JsonStringReader(ConvertibleJson, new AdHocSerializer { Features = Features.SerializeEnumsAsInteger });			

			var expected = ConvertibleText;
			var actual = target.Deserialize(reader) as MyConvertibleType;

			Assert.IsNotNull(actual);
			Assert.AreEqual(expected, actual.Value);
		}


		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		public void SerializeTest()
		{
			var contract = ConvertibleContract;
			var value = new MyConvertibleType { Value = ConvertibleText };
			var convertibleType = contract.Root;
			var target = new ConvertibleSerializer<MyConvertibleType>(convertibleType);			
			var writer = new TokenListJsonWriter();

			target.Serialize(writer, value);

			var expected = new Token[] { new Token(JsonToken.StringLiteral, value.Value) };

			Assert.IsNotNull(writer.Tokens);
			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}
	}
}
