﻿using Sencha.Serialization.Serializers;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Sencha.Serialization.Contracts;
using Sencha.Serialization;
using System.Runtime.Serialization;

using Token = Sencha.Serialization.Tests.TokenListJsonWriter.Token;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for CustomSerializationStrategyTest and is intended
	///to contain all CustomSerializationStrategyTest Unit Tests
	///</summary>
	[TestClass()]
	public class CustomSerializerTest
	{
		[DataContract]
		public class CustomSerializableObject : IJsonSerializable
		{
			public int IntField;
			public int IntProperty { get; set; }

			public CustomSerializableObject()
			{
				this.IntField = 10;
				this.IntProperty = 20;
			}

			#region IJsonSerializable Members

            public void ReadJson(IJsonReader reader)
			{
				reader.ReadObjectBegin();
				this.IntField = reader.ReadMember<int>("IntField");
				this.IntProperty = reader.ReadMember<int>("IntProperty");
				reader.ReadObjectEnd(false);
			}

            public void WriteJson(IJsonWriter writer)
			{
				writer.WriteObjectBegin();
				writer.WriteMember("IntField", this.IntField);
				writer.WriteMember("IntProperty", this.IntProperty);
				writer.WriteObjectEnd();
			}

			#endregion
		}

		public DataContract CustomObjectContract = new DataContract(typeof(CustomSerializableObject), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public string CustomObjectContractJson = "{ IntField: 40, IntProperty: 70 }";

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		/// <summary>
		///A test for CustomSerializationStrategy Constructor
		///</summary>
		[TestMethod()]
		public void CustomSerializationStrategyConstructorTest()
		{
			var objectType = CustomObjectContract.Root;
			var target = new CustomSerializer<CustomSerializableObject>();
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeTest()
		{
			var contract = CustomObjectContract;
			var objectType = contract.Root;
			var target = new CustomSerializer<CustomSerializableObject>();
			var context = new JsonSerializer_Accessor(contract);
			context.IgnoreCustomSerialization = false;
			var reader = new JsonStringReader(CustomObjectContractJson);

			var expected = new CustomSerializableObject { IntField = 40, IntProperty = 70 };
			var actual = target.Deserialize(reader) as CustomSerializableObject;

			Assert.IsNotNull(actual);
			Assert.AreEqual(expected.IntField, actual.IntField);
			Assert.AreEqual(expected.IntProperty, actual.IntProperty);
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeTest()
		{
			var contract = CustomObjectContract;
			var objectType = contract.Root;
			var target = new CustomSerializer<CustomSerializableObject>();
			var writer = new TokenListJsonWriter(new AdHocSerializer { Features = 0 });

			var value = new CustomSerializableObject { IntField = 40, IntProperty = 70 };
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
					new Token(JsonToken.Member, (JsonMember)"IntField"), 
					new Token(JsonToken.Number, value.IntField), 
					new Token(JsonToken.Member, (JsonMember)"IntProperty"), 
					new Token(JsonToken.Number, value.IntProperty), 
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}

	}
}
