﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Sencha.Serialization;
using Sencha.Serialization.Contracts;
using Sencha.Serialization.Serializers;
using System.Collections.Generic;
using System.Runtime.Serialization;

using Token = Sencha.Serialization.Tests.TokenListJsonWriter.Token;


namespace Sencha.Serialization.Tests
{
	/// <summary>
	///This is a test class for PlainObjectSerializationStrategyTest and is intended
	///to contain all PlainObjectSerializationStrategyTest Unit Tests
	///</summary>
	[TestClass()]
	public class DataContractObjectSerializerTest
	{
		public class PlainObject
		{
			public int IntField;
			public int IntProperty { get; set; }

			public PlainObject()
			{
				this.IntField = 10;
				this.IntProperty = 20;
			}
		}
		[DataContract]
		public class PlainWithOrder
		{
			[DataMember(Order = 3)]
			public int ThirdField;
			[DataMember(Order = 1)]
			public int FirstField;
			[DataMember(Order = 2)]
			public int SecondField;

			public PlainWithOrder()
			{

			}
		}

		[DataContract]
		public class WithContractPlainWithImplicitHidden
		{
			[DataMember(Order = 3)]
			public int VisibleField3;
			[DataMember(Order = 1)]
			public int VisibleField1;

			public int HiddenField2;

			public WithContractPlainWithImplicitHidden()
			{
				VisibleField3 = 3;
				VisibleField1 = 1;
				HiddenField2 = 2;
			}
		}
		[DataContract]
		public class WithContractPlainWithExplicitHidden
		{
			public int VisibleField3;
			public int VisibleField1;
			[IgnoreDataMember]
			public int HiddenField2;

			public WithContractPlainWithExplicitHidden()
			{
				VisibleField1 = 1;
				VisibleField3 = 3;
				HiddenField2 = 2;
			}
		}
		[DataContract]
		public class WithContractPlainWithMixedVisibility
		{
			[DataMember(Name = "VisibleFieldChanged")]
			public int VisibleField3;
			public int VisibleField1;
			[IgnoreDataMember]
			public int HiddenField2;

			public WithContractPlainWithMixedVisibility()
			{
				VisibleField1 = 1;
				VisibleField3 = 3;
				HiddenField2 = 2;
			}
		}

		public class NoContractPlainWithImplicitHidden
		{
			[DataMember(Order = 3)]
			public int VisibleField3;
			[DataMember(Order = 1)]
			public int VisibleField1;

			public int HiddenField2;

			public NoContractPlainWithImplicitHidden()
			{
				VisibleField3 = 3;
				VisibleField1 = 1;
				HiddenField2 = 2;
			}
		}
		public class NoContractPlainWithExplicitHidden
		{
			public int VisibleField3;
			public int VisibleField1;
			[IgnoreDataMember]
			public int HiddenField2;

			public NoContractPlainWithExplicitHidden()
			{
				VisibleField1 = 1;
				VisibleField3 = 3;
				HiddenField2 = 2;
			}
		}
		public class NoContractPlainWithMixedVisibility
		{
			[DataMember(Name = "VisibleFieldChanged")]
			public int VisibleField3;
			public int VisibleField1;
			[IgnoreDataMember]
			public int HiddenField2;

			public NoContractPlainWithMixedVisibility()
			{
				VisibleField1 = 1;
				VisibleField3 = 3;
				HiddenField2 = 2;
			}
		}

		public DataContract ObjectContract = new DataContract(typeof(object), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract PlainWithOrderContract = new DataContract(typeof(PlainWithOrder), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract PlainObjectContract = new DataContract(typeof(PlainObject), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);

		public DataContract WithContractPlainWithImplicitHiddenContract = new DataContract(typeof(WithContractPlainWithImplicitHidden), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract WithContractPlainWithExplicitHiddenContract = new DataContract(typeof(WithContractPlainWithExplicitHidden), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract WithContractPlainWithMixedVisibilityContract = new DataContract(typeof(WithContractPlainWithMixedVisibility), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract NoContractPlainWithImplicitHiddenContract = new DataContract(typeof(NoContractPlainWithImplicitHidden), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract NoContractPlainWithExplicitHiddenContract = new DataContract(typeof(NoContractPlainWithExplicitHidden), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract NoContractPlainWithMixedVisibilityContract = new DataContract(typeof(NoContractPlainWithMixedVisibility), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);

		public string PlainbjectContractJson = "{ \"IntField\": 40, \"IntProperty\": 70 }";

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}


		/// <summary>
		///A test for ObjectSerializationStrategy Constructor
		///</summary>
		[TestMethod()]
		public void PlainObjectSerializationStrategyConstructorTest()
		{
			var contract = PlainObjectContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<PlainObject>(PlainObjectContract, objectType);
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeTest()
		{
			var contract = PlainObjectContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<PlainObject>(PlainObjectContract, objectType);
			var context = new JsonSerializer_Accessor(contract);
			var reader = new JsonStringReader(PlainbjectContractJson);

			var expected = new PlainObject { IntField = 40, IntProperty = 70 };
			var actual = target.Deserialize(reader) as PlainObject;

			Assert.IsNotNull(actual);
			Assert.AreEqual(expected.IntField, actual.IntField);
			Assert.AreEqual(expected.IntProperty, actual.IntProperty);
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeTest()
		{
			var contract = PlainObjectContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<PlainObject>(PlainObjectContract, objectType);
			var context = new JsonSerializer_Accessor(contract);
			var writer = new TokenListJsonWriter(context);

			var value = new PlainObject { IntField = 40, IntProperty = 70 };
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
					new Token(JsonToken.Member,  (JsonMember)"IntField"), 
					new Token(JsonToken.Number, value.IntField), 
					new Token(JsonToken.Member,  (JsonMember)"IntProperty"), 
					new Token(JsonToken.Number, value.IntProperty), 
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeOrderedTest()
		{
			var contract = PlainWithOrderContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<PlainWithOrder>(PlainWithOrderContract, objectType);
			var context = new JsonSerializer_Accessor(contract);
			var writer = new TokenListJsonWriter(context);

			var value = new PlainWithOrder { FirstField = 1, SecondField = 2, ThirdField = 3 };
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
					new Token(JsonToken.Member,  (JsonMember)"FirstField"), 
					new Token(JsonToken.Number, value.FirstField), 
					new Token(JsonToken.Member,  (JsonMember)"SecondField"), 
					new Token(JsonToken.Number, value.SecondField), 
					new Token(JsonToken.Member,  (JsonMember)"ThirdField"), 
					new Token(JsonToken.Number, value.ThirdField), 
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeWithContractPlainWithImplicitHiddenTest()
		{
			var contract = WithContractPlainWithImplicitHiddenContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<WithContractPlainWithImplicitHidden>(WithContractPlainWithImplicitHiddenContract, objectType);
			var context = new JsonSerializer_Accessor(contract);
			var writer = new TokenListJsonWriter(context);

			var value = new WithContractPlainWithImplicitHidden();
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
					new Token(JsonToken.Member,  (JsonMember)"VisibleField1"), 
					new Token(JsonToken.Number, value.VisibleField1), 
					new Token(JsonToken.Member,  (JsonMember)"VisibleField3"), 
					new Token(JsonToken.Number, value.VisibleField3), 
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeWithContractPlainWithExplicitHiddenTest()
		{
			var contract = WithContractPlainWithExplicitHiddenContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<WithContractPlainWithExplicitHidden>(WithContractPlainWithExplicitHiddenContract, objectType);
			var context = new JsonSerializer_Accessor(contract);
			var writer = new TokenListJsonWriter(context);

			var value = new WithContractPlainWithExplicitHidden();
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeWithContractPlainWithMixedVisibilityTest()
		{
			var contract = WithContractPlainWithMixedVisibilityContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<WithContractPlainWithMixedVisibility>(WithContractPlainWithMixedVisibilityContract, objectType);
			var context = new JsonSerializer_Accessor(contract);
			var writer = new TokenListJsonWriter(context);

			var value = new WithContractPlainWithMixedVisibility();
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
					new Token(JsonToken.Member,  (JsonMember)"VisibleFieldChanged"), 
					new Token(JsonToken.Number, value.VisibleField3), 
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeNoContractPlainWithImplicitHiddenTest()
		{
			var contract = NoContractPlainWithImplicitHiddenContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<NoContractPlainWithImplicitHidden>(NoContractPlainWithImplicitHiddenContract, objectType);
			var context = new JsonSerializer_Accessor(contract);
			var writer = new TokenListJsonWriter(context);

			var value = new NoContractPlainWithImplicitHidden();
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
					new Token(JsonToken.Member,  (JsonMember)"VisibleField1"), 
					new Token(JsonToken.Number, value.VisibleField1), 
					new Token(JsonToken.Member,  (JsonMember)"HiddenField2"), 
					new Token(JsonToken.Number, value.HiddenField2), 
					new Token(JsonToken.Member,  (JsonMember)"VisibleField3"), 
					new Token(JsonToken.Number, value.VisibleField3), 
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeNoContractPlainWithExplicitHiddenTest()
		{
			var contract = NoContractPlainWithExplicitHiddenContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<NoContractPlainWithExplicitHidden>(NoContractPlainWithExplicitHiddenContract, objectType);
			var context = new JsonSerializer_Accessor(contract);
			var writer = new TokenListJsonWriter(context);

			var value = new NoContractPlainWithExplicitHidden();
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
					new Token(JsonToken.Member, (JsonMember)"VisibleField3"), 
					new Token(JsonToken.Number, value.VisibleField3), 
					new Token(JsonToken.Member,  (JsonMember)"VisibleField1"), 
					new Token(JsonToken.Number, value.VisibleField1), 
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeNoContractPlainWithMixedVisibilityTest()
		{
			var contract = NoContractPlainWithMixedVisibilityContract;
			var objectType = contract.Root;
			var target = new ObjectSerializer<NoContractPlainWithMixedVisibility>(NoContractPlainWithMixedVisibilityContract, objectType);
			var context = new JsonSerializer_Accessor(contract);
			var writer = new TokenListJsonWriter(context);

			var value = new NoContractPlainWithMixedVisibility();
			target.Serialize(writer, value);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
					new Token(JsonToken.Member,  (JsonMember)"VisibleField3"), 
					new Token(JsonToken.Number, value.VisibleField3), 
					new Token(JsonToken.Member,  (JsonMember)"VisibleField1"), 
					new Token(JsonToken.Number, value.VisibleField1),
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}
	}
}
