﻿
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Sencha.Serialization.Contracts;
using Sencha.Serialization.Serializers;
using Sencha.Serialization;

using Token = Sencha.Serialization.Tests.TokenListJsonWriter.Token;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for DictionarySerializationStrategyTest and is intended
	///to contain all DictionarySerializationStrategyTest Unit Tests
	///</summary>
	[TestClass()]
	public class DictionarySerializerTest
	{
		private DataContract HashtableContract = new DataContract(typeof(Hashtable), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private DataContract StringIntDictionaryContract = new DataContract(typeof(Dictionary<string, int>), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private DataContract IntStringDictionaryContract = new DataContract(typeof(Dictionary<int, string>), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private string IntIntDictionaryJson = "{ \"1\":1045, \"2\":87321, \"4\":1000 }";
		private string StringIntDictionaryJson = "{ \"label\":1045, \"otherLabel\":87321, labelNoQuotes:1000 }";
		private string IntStringDictionaryJson = "{ \"1045\":\"1045\", \"87321\":\"otherLabel\", \"1000\":\"labelNoQuotes\" }";

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for DictionarySerializationStrategy Constructor
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DictionarySerializationStrategyConstructorTest()
		{
			TypeDescription dictionaryType = HashtableContract.Root;
			var target = new DictionarySerializer<Hashtable>(dictionaryType);

			dictionaryType = StringIntDictionaryContract.Root;
			var target1 = new DictionarySerializer<Dictionary<string, int>>(dictionaryType);

			dictionaryType = IntStringDictionaryContract.Root;
			var target2 = new DictionarySerializer<Dictionary<int, string>>(dictionaryType);
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeDictionaryTest()
		{
			var contract = HashtableContract;
			var dictionaryType = HashtableContract.Root;
			var target = new DictionarySerializer<Hashtable>(dictionaryType);
			var reader = new JsonStringReader(IntIntDictionaryJson);


			IDictionary actual;
			actual = target.Deserialize(reader) as IDictionary;

			Assert.IsNotNull(actual);
			Assert.AreEqual(3, actual.Count, "wrong dictionary size");
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeIntStringDictionaryTest()
		{
			var contract = IntStringDictionaryContract;
			var dictionaryType = IntStringDictionaryContract.Root;
			var target = new DictionarySerializer<Dictionary<int, string>>(dictionaryType);
			var reader = new JsonStringReader(IntStringDictionaryJson);

			IDictionary<int, string> actual;
			actual = target.Deserialize(reader) as IDictionary<int, string>;

			Assert.IsNotNull(actual);
			Assert.AreEqual(3, actual.Count, "wrong dictionary size");
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void DeserializeStringIntDictionaryTest()
		{
			var contract = StringIntDictionaryContract;
			var dictionaryType = StringIntDictionaryContract.Root;
			var target = new DictionarySerializer<Dictionary<string, int>>(dictionaryType);
			var reader = new JsonStringReader(StringIntDictionaryJson);

			IDictionary<string, int> actual;
			actual = target.Deserialize(reader) as IDictionary<string, int>;

			Assert.IsNotNull(actual);
			Assert.AreEqual(3, actual.Count, "wrong dictionary size");
		}

		///// <summary>
		/////A test for PopulateDictionary
		/////</summary>
		//[TestMethod()]
		//[DeploymentItem("Sencha.Serialization.dll")]
		//public void PopulateDictionaryTest()
		//{
		//    var target = new DictionarySerializationStrategy_Accessor<Hashtable>(HashtableContract.Root);
		//    Dictionary<string, object> source = new Dictionary<string, object> { { "1", 1m }, { "2", new DateTime() }, { "3", "3" }, { "4", null } };
		//    IFormatProvider format = System.Globalization.CultureInfo.InvariantCulture;

		//    IDictionary actual;
		//    actual = target.PopulateDictionary(source, format) as IDictionary;

		//    Assert.IsInstanceOfType(actual, HashtableContract.Root.GetClrType());
		//    foreach (var key in source.Keys)
		//    {
		//        Assert.IsTrue(actual.Contains(key), "populated dictionary does not contains one of key");
		//        Assert.AreEqual(source[key], actual[key], "populated dictionary has wrong value");
		//    }
		//}

		///// <summary>
		/////A test for PopulateDictionary
		/////</summary>
		//[TestMethod()]
		//[DeploymentItem("Sencha.Serialization.dll")]
		//public void PopulateGenericDictionaryTest()
		//{
		//    var target = new DictionarySerializationStrategy_Accessor<Dictionary<int, string>>(IntStringDictionaryContract.Root);
		//    Dictionary<string, object> source = new Dictionary<string, object> { { "1", 1m }, { "2", new DateTime() }, { "3", "3" }, { "4", null } };
		//    IFormatProvider format = System.Globalization.CultureInfo.InvariantCulture;
		//    IDictionary<int, string> actual;
		//    actual = target.PopulateDictionary(source, format) as IDictionary<int, string>;

		//    Assert.IsInstanceOfType(actual, IntStringDictionaryContract.Root.GetClrType());
		//    foreach (var key in source.Keys)
		//    {
		//        var intKey = int.Parse(key);
		//        Assert.IsTrue(actual.ContainsKey(intKey), "populated dictionary does not contains one of key");
		//        Assert.AreEqual(source[key] == null ? null : Convert.ToString(source[key], format), actual[intKey], "populated dictionary has wrong value");
		//    }
		//}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		[DeploymentItem("Sencha.Serialization.dll")]
		public void SerializeTest()
		{
			var contract = HashtableContract;
			var dictionaryType = HashtableContract.Root;
			var target = new DictionarySerializer<Hashtable>(dictionaryType);
			var writer = new TokenListJsonWriter();
			var dictionary = new Hashtable { { 1, 1m }, { 2, 1.0f }, { 3, "3" } }; ;

			target.Serialize(writer, dictionary);

			var expected = new[] { 
				new Token(JsonToken.BeginObject), 
					new Token(JsonToken.Member, (JsonMember)"3"),
					new Token(JsonToken.StringLiteral, "3"), 	
					new Token(JsonToken.Member, (JsonMember)"2"),
					new Token(JsonToken.Number, 1.0f), 
					new Token(JsonToken.Member, (JsonMember)"1"),
					new Token(JsonToken.Number, 1m), 					
				new Token(JsonToken.EndOfObject) 
			};

			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}
	}
}
