﻿extern alias json;
extern alias bson;

using Sencha.Serialization.Serializers;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;

using Sencha.Serialization.Contracts;
using System.Collections.Generic;
using Sencha.Serialization;
using System.Runtime.Serialization;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for InheritedObjectSerializationStrategyTest and is intended
	///to contain all InheritedObjectSerializationStrategyTest Unit Tests
	///</summary>
	[TestClass()]
	public class ObjectSerializerTest
	{
		[DataContract]
		[KnownType(typeof(InheritedObjectA))]
		[KnownType(typeof(InheritedObjectB))]
		public class BaseObject
		{
			[DataMember]
			public int VisibleField2;
			[DataMember]
			public int VisibleField1;
			[DataMember]
			public int ShadowedVisibleField;
			[DataMember]
			public virtual int OverridenVisibleProperty { get; set; }

			public int HiddenField;

			public BaseObject()
			{
				VisibleField1 = 1;
				VisibleField2 = 2;
				HiddenField = 1;
			}
		}
		[DataContract]
		public class InheritedObjectA : BaseObject
		{
			[DataMember]
			public int IntField;
			[DataMember]
			public string StringField;
			[DataMember]
			new public int ShadowedVisibleField;
			[DataMember]
			public override int OverridenVisibleProperty { get; set; }
			[DataMember]
			public int[] IntArray { get; set; }
			[DataMember]
			public string[] StringArray { get; set; }
			[DataMember]
			public InheritedObjectB ObjectField { get; set; }

			new public int HiddenField;

			public InheritedObjectA()
			{
				IntField = 3;
				StringField = "str";
				HiddenField = 2;
				IntArray = new int[0];
				StringArray = new string[0];
				ObjectField = new InheritedObjectB();
			}
		}
		[DataContract]
		public class InheritedObjectB : BaseObject
		{
			[DataMember(Name = "VisibleFieldChanged")]
			public int VisibleField7;
			[DataMember]
			public int VisibleField8;
			[DataMember]
			new public int ShadowedVisibleField;
			[DataMember]
			public override int OverridenVisibleProperty { get; set; }

			new public int HiddenField;

			public InheritedObjectB()
			{
				VisibleField7 = 3;
				VisibleField8 = 4;
				HiddenField = 2;
			}
		}

		private TestContext testContextInstance;

		public DataContract InheritedObjectAContract = new DataContract(typeof(InheritedObjectA), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract InheritedObjectBContract = new DataContract(typeof(InheritedObjectB), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		public DataContract BaseObjectContract = new DataContract(typeof(BaseObject), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);


		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		/// <summary>
		///A test for ObjectSerializationStrategy Constructor
		///</summary>
		[TestMethod()]
		public void InheritedObjectSerializationStrategyConstructorTest()
		{
			var contract = InheritedObjectBContract;
			var objectType = InheritedObjectBContract.Root;
			var target = new ObjectSerializer<InheritedObjectB>(contract, objectType);
		}

		private System.Reflection.MemberInfo GetMember<EntryT, ValueT>(System.Linq.Expressions.Expression<Func<EntryT, ValueT>> property)
		{
			try { return (property.Body as System.Linq.Expressions.MemberExpression).Member; }
			catch (NullReferenceException)
			{
				throw new ArgumentException("Expression shoud be System.Linq.Expressions.MemberExpression", "property");
			}
		}
	}
}
