﻿using Sencha.Serialization.Serializers;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using Sencha.Serialization.Contracts;
using Sencha.Serialization;
using System.IO;

using Token = Sencha.Serialization.Tests.TokenListJsonWriter.Token;

namespace Sencha.Serialization.Tests
{
    
    
    /// <summary>
    ///This is a test class for StreamSerializationStrategyTest and is intended
    ///to contain all StreamSerializationStrategyTest Unit Tests
    ///</summary>
	[TestClass()]
	public class StreamSerializerTest
	{
		private DataContract StreamContract = new DataContract(typeof(Stream), MetadataInterpreter.DataContract, MetadataDescriber.ReflectionPublicOnly);
		private string OriginalString = "Man is distinguished, not only by his reason, but by this singular passion from other animals, which is a lust of the mind, that by a perseverance of delight in the continued and indefatigable generation of knowledge, exceeds the short vehemence of any carnal pleasure.";

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}


		/// <summary>
		///A test for StreamSerializationStrategy Constructor
		///</summary>
		[TestMethod()]
		public void StreamSerializationStrategyConstructorTest()
		{
			var streamType = StreamContract.Root;
			StreamSerializer target = new StreamSerializer(streamType);
		}

		/// <summary>
		///A test for Deserialize
		///</summary>
		[TestMethod()]
		public void DeserializeTest()
		{
			var contract = StreamContract;
			var encoding = System.Text.Encoding.UTF8;
			var bytes = encoding.GetBytes(OriginalString);
			var base64String = "\"" + Convert.ToBase64String(bytes) + "\"";
			var streamType = contract.Root;
			var target = new StreamSerializer(streamType);
			var context = new JsonSerializer_Accessor(contract);
			var reader = new JsonStringReader(base64String);
			//context.m_reader = reader;

			var expected = bytes;
			var actualStream = target.Deserialize(reader) as Stream;
			Assert.IsNotNull(actualStream);
			var actual = new byte[actualStream.Length];
			actualStream.Read(actual, 0, actual.Length);

			Assert.IsNotNull(actual);
			Assert.AreEqual(expected.Length, actual.Length, "arrays has different length");
			for (var i = 0; i < actual.Length; i++)
				Assert.AreEqual(expected[i], actual[i], "arrays not same");
		}

		/// <summary>
		///A test for Serialize
		///</summary>
		[TestMethod()]
		public void SerializeTest()
		{
			var contract = StreamContract;
			var encoding = System.Text.Encoding.UTF8;
			var bytes = encoding.GetBytes(OriginalString);
			var stream = new MemoryStream(bytes);
			var base64String = Convert.ToBase64String(bytes);
			var byteArrayType = contract.Root;
			var target = new StreamSerializer(byteArrayType);			
			var writer = new TokenListJsonWriter();

			target.Serialize(writer, stream);

			var expected = new Token[] { new Token(JsonToken.StringLiteral, base64String) };

			Assert.IsNotNull(expected);
			Assert.AreEqual(expected.Length, writer.Tokens.Count, "wrong markup generated");
			for (int i = 0; i < expected.Length; i++)
			{
				var actualToken = writer.Tokens[i];
				var expectedToken = expected[i];

				Assert.AreEqual(expectedToken.JsToken, actualToken.JsToken, "unexpected token generated");
				Assert.AreEqual(expectedToken.JsValue, actualToken.JsValue, "unexpected value generated");
			}
		}
	}
}
