﻿using Sencha.Serialization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for JsonUtilsTest and is intended
	///to contain all JsonUtilsTest Unit Tests
	///</summary>
	[TestClass()]
	public class JsonUtilsTest
	{
		private const int PerfomanceRuns = 20000;
		private const int BufferSize = 400;

		private TestContext testContextInstance;

		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		#region Additional test attributes
		// 
		//You can use the following additional attributes as you write your tests:
		//
		//Use ClassInitialize to run code before running the first test in the class
		//[ClassInitialize()]
		//public static void MyClassInitialize(TestContext testContext)
		//{
		//}
		//
		//Use ClassCleanup to run code after all tests in a class have run
		//[ClassCleanup()]
		//public static void MyClassCleanup()
		//{
		//}
		//
		//Use TestInitialize to run code before running each test
		//[TestInitialize()]
		//public void MyTestInitialize()
		//{
		//}
		//
		//Use TestCleanup to run code after each test has run
		//[TestCleanup()]
		//public void MyTestCleanup()
		//{
		//}
		//
		#endregion


		/// <summary>
		///A test for UInt64ToBuffer
		///</summary>
		[TestMethod()]
		public void UInt64ToBufferMaxTest()
		{
			ulong uvalue = ulong.MaxValue;
			char[] buffer = new char[BufferSize];
			int start = 0;
			char[] expectedChars = uvalue.ToString().ToCharArray();
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.UInt64ToBuffer(uvalue, buffer, start, InvariantFormat);
			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for UInt64ToBuffer
		///</summary>
		[TestMethod()]
		public void UInt64ToBufferMinTest()
		{
			ulong uvalue = ulong.MinValue;
			char[] buffer = new char[BufferSize];
			int start = 0;
			char[] expectedChars = uvalue.ToString().ToCharArray();
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.UInt64ToBuffer(uvalue, buffer, start, InvariantFormat);
			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for UInt64ToBuffer
		///</summary>
		[TestMethod()]
		public void UInt64ToBufferTest()
		{
			ulong uvalue = ulong.MaxValue / 2;
			char[] buffer = new char[BufferSize];
			int start = 0;
			char[] expectedChars = uvalue.ToString().ToCharArray();
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.UInt64ToBuffer(uvalue, buffer, start, InvariantFormat);
			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}


		/// <summary>
		///A test for UInt32ToHexBuffer
		///</summary>
		[TestMethod()]
		public void UInt32ToHexBufferMaxTest()
		{
			uint uvalue = uint.MaxValue;
			char[] buffer = new char[BufferSize];
			int start = 0;
			char[] expectedChars = uvalue.ToString("X", InvariantFormat).ToCharArray();
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.UInt32ToHexBuffer(uvalue, buffer, start);
			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for UInt32ToHexBuffer
		///</summary>
		[TestMethod()]
		public void UInt32ToHexBufferMinTest()
		{
			uint uvalue = uint.MinValue;
			char[] buffer = new char[BufferSize];
			int start = 0;
			char[] expectedChars = uvalue.ToString("X", InvariantFormat).ToCharArray();
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.UInt32ToHexBuffer(uvalue, buffer, start);
			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for UInt32ToHexBuffer
		///</summary>
		[TestMethod()]
		public void UInt32ToHexBufferTest()
		{
			uint uvalue = uint.MaxValue / 2;
			char[] buffer = new char[BufferSize];
			int start = 0;
			char[] expectedChars = uvalue.ToString("X", InvariantFormat).ToCharArray();
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.UInt32ToHexBuffer(uvalue, buffer, start);
			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for UInt32ToBuffer
		///</summary>
		[TestMethod()]
		public void UInt32ToBufferMaxTest()
		{
			uint uvalue = uint.MaxValue;
			char[] buffer = new char[BufferSize];
			int start = 0;
			char[] expectedChars = uvalue.ToString().ToCharArray();
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.UInt32ToBuffer(uvalue, buffer, start, InvariantFormat);
			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for UInt32ToBuffer
		///</summary>
		[TestMethod()]
		public void UInt32ToBufferMinTest()
		{
			uint uvalue = uint.MinValue;
			char[] buffer = new char[BufferSize];
			int start = 0;
			char[] expectedChars = uvalue.ToString().ToCharArray();
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.UInt32ToBuffer(uvalue, buffer, start, InvariantFormat);
			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for UInt32ToBuffer
		///</summary>
		[TestMethod()]
		public void UInt32ToBufferTest()
		{
			uint uvalue = uint.MaxValue / 2;
			char[] buffer = new char[BufferSize];
			int start = 0;
			char[] expectedChars = uvalue.ToString().ToCharArray();
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.UInt32ToBuffer(uvalue, buffer, start, InvariantFormat);
			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for SingleToBuffer
		///</summary>
		[TestMethod()]
		public void SingleToBufferMaxTest()
		{
			float value = float.MaxValue;
			var expectedChars = value.ToString("R", InvariantFormat).ToCharArray();
			int prec = 9;
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.SingleToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for SingleToBuffer
		///</summary>
		[TestMethod()]
		public void SingleToBufferMinTest()
		{
			float value = float.MinValue;
			var expectedChars = value.ToString("R", InvariantFormat).ToCharArray();
			int prec = 9;
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.SingleToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for SingleToBuffer
		///</summary>
		[TestMethod()]
		public void SingleToBufferZeroTest()
		{
			float value = 0;
			var expectedChars = value.ToString("R", InvariantFormat).ToCharArray();
			int prec = 9;
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.SingleToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for SingleToBuffer
		///</summary>
		[TestMethod()]
		public void SingleToBufferTest()
		{
			float value = -1213213.213213123f;
			var expectedChars = value.ToString("R", InvariantFormat).ToCharArray();
			int prec = 9;
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.SingleToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}


		/// <summary>
		///A test for DoubleToBuffer
		///</summary>
		[TestMethod()]
		public void DoubleToBufferMaxTest()
		{
			double value = double.MaxValue;
			var expectedChars = value.ToString("R", InvariantFormat).ToCharArray();
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.DoubleToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}


		/// <summary>
		///A test for DoubleToBuffer
		///</summary>
		[TestMethod()]
		public void DoubleToBufferMinTest()
		{
			double value = double.MinValue;
			var expectedChars = value.ToString("R", InvariantFormat).ToCharArray();
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.DoubleToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}


		/// <summary>
		///A test for DoubleToBuffer
		///</summary>
		[TestMethod()]
		public void DoubleToBufferZeroTest()
		{
			double value = 0;
			var expectedChars = value.ToString("R", InvariantFormat).ToCharArray();
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.DoubleToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}
		/// <summary>
		///A test for DoubleToBuffer
		///</summary>
		[TestMethod()]
		public void DoubleToBufferTest()
		{
			double value = -1213213.213213123f;
			var expectedChars = value.ToString("R", InvariantFormat).ToCharArray();
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.DoubleToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}


		/// <summary>
		///A test for Int64ToBuffer
		///</summary>
		[TestMethod()]
		public void Int64ToBufferMaxTest()
		{
			long value = long.MaxValue;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int64ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for Int64ToBuffer
		///</summary>
		[TestMethod()]
		public void Int64ToBufferMinTest()
		{
			long value = long.MinValue;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int64ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for Int64ToBuffer
		///</summary>
		[TestMethod()]
		public void Int64ToBufferZeroTest()
		{
			long value = 0;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int64ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for Int64ToBuffer
		///</summary>
		[TestMethod()]
		public void Int64ToBufferTest()
		{
			long value = long.MaxValue / 2;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int64ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for Int64ToBuffer
		///</summary>
		[TestMethod()]
		public void Int64ToBufferNegativeTest()
		{
			long value = long.MinValue / 2;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int64ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for Int32ToBuffer
		///</summary>
		[TestMethod()]
		public void Int32ToBufferMaxTest()
		{
			int value = int.MaxValue;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int32ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for Int32ToBuffer
		///</summary>
		[TestMethod()]
		public void Int32ToBufferMinTest()
		{
			int value = int.MinValue;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int32ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for Int32ToBuffer
		///</summary>
		[TestMethod()]
		public void Int32ToBufferZeroTest()
		{
			int value = 0;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int32ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}
		/// <summary>
		///A test for Int32ToBuffer
		///</summary>
		[TestMethod()]
		public void Int32ToBufferTest()
		{
			int value = int.MaxValue / 2;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int32ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}
		/// <summary>
		///A test for Int32ToBuffer
		///</summary>
		[TestMethod()]
		public void Int32ToBufferNegativeTest()
		{
			int value = int.MinValue / 2;
			char[] buffer = new char[BufferSize];
			var expectedChars = value.ToString(InvariantFormat).ToCharArray();
			int start = 0;
			int expected = expectedChars.Length;
			int actual;
			actual = JsonUtils.Int32ToBuffer(value, buffer, start, InvariantFormat);

			Assert.AreEqual(expected, actual);
			for (int i = start; i < expectedChars.Length + start; i++)
			{
				Assert.AreEqual(expectedChars[i], buffer[i]);
			}
		}

		/// <summary>
		///A test for StringToUInt64
		///</summary>
		[TestMethod()]
		public void StringToUInt64MaxTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			ulong expected = ulong.MaxValue;
			ulong actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToUInt64(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToUInt64
		///</summary>
		[TestMethod()]
		public void StringToUInt64MinTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			ulong expected = ulong.MinValue;
			ulong actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToUInt64(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToUInt64
		///</summary>
		[TestMethod()]
		public void StringToUInt64Test()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			ulong expected = ulong.MaxValue / 2;
			ulong actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToUInt64(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for StringToUInt32
		///</summary>
		[TestMethod()]
		public void StringToUInt32MaxTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			uint expected = uint.MaxValue;
			uint actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToUInt32(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToUInt32
		///</summary>
		[TestMethod()]
		public void StringToUInt32MinTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			uint expected = uint.MinValue;
			uint actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToUInt32(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for StringToUInt32
		///</summary>
		[TestMethod()]
		public void StringToUInt32Test()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			uint expected = uint.MaxValue / 2;
			uint actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToUInt32(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToInt64
		///</summary>
		[TestMethod()]
		public void StringToInt64MaxTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			long expected = long.MaxValue;
			long actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt64(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToInt64
		///</summary>
		[TestMethod()]
		public void StringToInt64MinTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			long expected = long.MinValue;
			long actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt64(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToInt64
		///</summary>
		[TestMethod()]
		public void StringToInt64ZeroTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			long expected = 0;
			long actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt64(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToInt64
		///</summary>
		[TestMethod()]
		public void StringToInt64Test()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			long expected = long.MaxValue / 2;
			long actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt64(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToInt64
		///</summary>
		[TestMethod()]
		public void StringToInt64NegativeTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			long expected = long.MinValue / 2;
			long actual;
			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt64(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToInt32
		///</summary>
		[TestMethod()]
		public void StringToInt32MaxTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = int.MaxValue;
			int actual;

			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt32(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToInt32
		///</summary>
		[TestMethod()]
		public void StringToInt32MinTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = int.MinValue;
			int actual;

			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt32(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToInt32
		///</summary>
		[TestMethod()]
		public void StringToInt32Test()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = int.MaxValue / 2;
			int actual;

			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt32(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToInt32
		///</summary>
		[TestMethod()]
		public void StringToInt32NegativeTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = int.MinValue / 2;
			int actual;

			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt32(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}
		/// <summary>
		///A test for StringToInt32
		///</summary>
		[TestMethod()]
		public void StringToInt32ZeroTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			int expected = 0;
			int actual;

			var valueChars = expected.ToString().ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToInt32(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToFloat
		///</summary>
		[TestMethod()]
		public void StringToFloatMaxTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			float expected = (float)Math.Round(float.MaxValue, 9);
			float actual;

			var valueChars = expected.ToString("R", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToFloat(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToFloat
		///</summary>
		[TestMethod()]
		public void StringToFloatMinTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			float expected = (float)Math.Round(float.MinValue, 9);
			float actual;

			var valueChars = expected.ToString("R", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToFloat(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToFloat
		///</summary>
		[TestMethod()]
		public void StringToFloatZeroTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			float expected = 0;
			float actual;

			var valueChars = expected.ToString("R", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToFloat(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToFloat
		///</summary>
		[TestMethod()]
		public void StringToFloatTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			float expected = -1213213.213213123f;
			float actual;

			var valueChars = expected.ToString("R", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToFloat(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToDouble
		///</summary>
		[TestMethod()]
		public void StringToDoubleMaxTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			double expected = Math.Round(double.MaxValue, 9);
			double actual;

			var valueChars = expected.ToString("R", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToDouble(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}
		/// <summary>
		///A test for StringToDouble
		///</summary>
		[TestMethod()]
		public void StringToDoubleMinTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			double expected = Math.Round(double.MinValue, 9);
			double actual;

			var valueChars = expected.ToString("R", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToDouble(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToDouble
		///</summary>
		[TestMethod()]
		public void StringToDoubleZeroTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			double expected = 0;
			double actual;

			var valueChars = expected.ToString("R", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToDouble(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for StringToDouble
		///</summary>
		[TestMethod()]
		public void StringToDoubleTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			double expected = -1213213.213213122;
			double actual;

			var valueChars = expected.ToString("R", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.StringToDouble(buffer, start, len, InvariantFormat);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for HexStringToUInt32
		///</summary>
		[TestMethod()]
		public void HexStringToUInt32MaxTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			uint expected = uint.MaxValue;
			uint actual;

			var valueChars = expected.ToString("X", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.HexStringToUInt32(buffer, start, len);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for HexStringToUInt32
		///</summary>
		[TestMethod()]
		public void HexStringToUInt32MinTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			uint expected = uint.MinValue;
			uint actual;

			var valueChars = expected.ToString("X", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.HexStringToUInt32(buffer, start, len);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for HexStringToUInt32
		///</summary>
		[TestMethod()]
		public void HexStringToUInt32ZeroTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			uint expected = 0;
			uint actual;

			var valueChars = expected.ToString("X", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.HexStringToUInt32(buffer, start, len);
			Assert.AreEqual(expected, actual);
		}


		/// <summary>
		///A test for HexStringToUInt32
		///</summary>
		[TestMethod()]
		public void HexStringToUInt32Test()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			uint expected = uint.MaxValue / 2;
			uint actual;

			var valueChars = expected.ToString("X", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.HexStringToUInt32(buffer, start, len);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for HexStringToUInt32
		///</summary>
		[TestMethod()]
		public void HexStringToUInt32NegativeTest()
		{
			char[] buffer = new char[BufferSize];
			int start = 0;
			uint expected = uint.MaxValue / 2;
			uint actual;

			var valueChars = expected.ToString("X", InvariantFormat).ToCharArray();
			int len = valueChars.Length;
			valueChars.CopyTo(buffer, start);

			actual = JsonUtils.HexStringToUInt32(buffer, start, len);
			Assert.AreEqual(expected, actual);
		}

		public System.Globalization.CultureInfo InvariantFormat { get { return System.Globalization.CultureInfo.InvariantCulture; } }



		private double[] GenerateDoubles(int length)
		{
			var r = new Random();
			var ds = new double[length];
			for (int i = 0; i < length; i++)
			{
				ds[i] = r.NextDouble() * r.Next(0, 10000000);
			}
			return ds;
		}
		private float[] GenerateSingles(int length)
		{
			var r = new Random();
			var ds = new float[length];
			for (int i = 0; i < length; i++)
			{
				ds[i] = (float)r.NextDouble() * r.Next(0, 1000000);
			}
			return ds;
		}
		private int[] GenerateInt32(int length)
		{
			var r = new Random();
			var ds = new int[length];
			for (int i = 0; i < length; i++)
			{
				ds[i] = r.Next(int.MinValue, int.MaxValue);
			}
			return ds;
		}
		private uint[] GenerateUInt32(int length)
		{
			var r = new Random();
			var ds = new uint[length];
			for (int i = 0; i < length; i++)
			{
				ds[i] = unchecked((uint)r.Next(int.MinValue, int.MaxValue));
			}
			return ds;
		}
		private long[] GenerateInt64(int length)
		{
			var r = new Random();
			var ds = new long[length];
			for (int i = 0; i < length; i++)
			{
				ds[i] = unchecked((long)(ulong.MaxValue * r.NextDouble()));
			}
			return ds;
		}
		private ulong[] GenerateUInt64(int length)
		{
			var r = new Random();
			var ds = new ulong[length];
			for (int i = 0; i < length; i++)
			{
				ds[i] = unchecked((ulong)(ulong.MaxValue * r.NextDouble()));
			}
			return ds;
		}

		/// <summary>
		///A test for DoubleToBuffer
		///</summary>
		[TestMethod()]
		public void Perf_DoubleToBufferTest()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var doubles = GenerateDoubles(PerfomanceRuns);

			nativeTicks.Start();
			for (var i = 0; i < doubles.Length; i++)
			{
				doubles[i].ToString("R", InvariantFormat);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < doubles.Length; i++)
			{
				JsonUtils.DoubleToBuffer(doubles[i], buffer, 0, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for HexStringToUInt32
		///</summary>
		[TestMethod()]
		public void Perf_HexStringToUInt32Test()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateUInt32(PerfomanceRuns);
			var intStrs = Array.ConvertAll(ints, i => i.ToString("X", InvariantFormat));
			var intBuffs = Array.ConvertAll(intStrs, s => s.ToCharArray());

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				UInt32.Parse(intStrs[i], System.Globalization.NumberStyles.HexNumber);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.HexStringToUInt32(intBuffs[i], 0, intBuffs[i].Length);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for Int32ToBuffer
		///</summary>
		[TestMethod()]
		public void Perf_Int32ToBufferTest()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateInt32(PerfomanceRuns);

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				ints[i].ToString(InvariantFormat);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.Int32ToBuffer(ints[i], buffer, 0, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for Int64ToBuffer
		///</summary>
		[TestMethod()]
		public void Perf_Int64ToBufferTest()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateInt64(PerfomanceRuns);

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				ints[i].ToString(InvariantFormat);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.Int64ToBuffer(ints[i], buffer, 0, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for SingleToBuffer
		///</summary>
		[TestMethod()]
		public void Perf_SingleToBufferTest()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var singles = GenerateSingles(PerfomanceRuns);

			nativeTicks.Start();
			for (var i = 0; i < singles.Length; i++)
			{
				singles[i].ToString("R", InvariantFormat);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < singles.Length; i++)
			{
				JsonUtils.SingleToBuffer(singles[i], buffer, 0, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for StringToDouble
		///</summary>
		[TestMethod()]
		public void Perf_StringToDoubleTest()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateDoubles(PerfomanceRuns);
			var intStrs = Array.ConvertAll(ints, i => i.ToString("R", InvariantFormat));
			var intBuffs = Array.ConvertAll(intStrs, s => s.ToCharArray());

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				Double.Parse(intStrs[i], InvariantFormat);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.StringToDouble(intBuffs[i], 0, intBuffs[i].Length, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for StringToFloat
		///</summary>
		[TestMethod()]
		public void Perf_StringToFloatTest()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateSingles(PerfomanceRuns);
			var intStrs = Array.ConvertAll(ints, i => i.ToString("R", InvariantFormat));
			var intBuffs = Array.ConvertAll(intStrs, s => s.ToCharArray());

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				Single.Parse(intStrs[i], InvariantFormat);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.StringToFloat(intBuffs[i], 0, intBuffs[i].Length, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for StringToInt32
		///</summary>
		[TestMethod()]
		public void Perf_StringToInt32Test()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateInt32(PerfomanceRuns);
			var intStrs = Array.ConvertAll(ints, i => i.ToString(InvariantFormat));
			var intBuffs = Array.ConvertAll(intStrs, s => s.ToCharArray());

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				Int32.Parse(intStrs[i]);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.StringToInt32(intBuffs[i], 0, intBuffs[i].Length, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for StringToInt64
		///</summary>
		[TestMethod()]
		public void Perf_StringToInt64Test()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateInt64(PerfomanceRuns);
			var intStrs = Array.ConvertAll(ints, i => i.ToString(InvariantFormat));
			var intBuffs = Array.ConvertAll(intStrs, s => s.ToCharArray());

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				Int64.Parse(intStrs[i]);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.StringToInt64(intBuffs[i], 0, intBuffs[i].Length, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for StringToUInt32
		///</summary>
		[TestMethod()]
		public void Perf_StringToUInt32Test()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateUInt32(PerfomanceRuns);
			var intStrs = Array.ConvertAll(ints, i => i.ToString(InvariantFormat));
			var intBuffs = Array.ConvertAll(intStrs, s => s.ToCharArray());

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				UInt32.Parse(intStrs[i]);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.StringToUInt32(intBuffs[i], 0, intBuffs[i].Length, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for StringToUInt64
		///</summary>
		[TestMethod()]
		public void Perf_StringToUInt64Test()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateUInt64(PerfomanceRuns);
			var intStrs = Array.ConvertAll(ints, i => i.ToString(InvariantFormat));
			var intBuffs = Array.ConvertAll(intStrs, s => s.ToCharArray());

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				UInt64.Parse(intStrs[i]);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.StringToUInt64(intBuffs[i], 0, intBuffs[i].Length, InvariantFormat);
			}
			customTicks.Stop();


			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for UInt32ToBuffer
		///</summary>
		[TestMethod()]
		public void Perf_UInt32ToBufferTest()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateUInt32(PerfomanceRuns);

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				ints[i].ToString(InvariantFormat);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.UInt32ToBuffer(ints[i], buffer, 0, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for UInt32ToHexBuffer
		///</summary>
		[TestMethod()]
		public void Perf_UInt32ToHexBufferTest()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateUInt32(PerfomanceRuns);

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				ints[i].ToString("X", InvariantFormat);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.UInt32ToHexBuffer(ints[i], buffer, 0);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for UInt64ToBuffer
		///</summary>
		[TestMethod()]
		public void Perf_UInt64ToBufferTest()
		{
			char[] buffer = new char[BufferSize];

			var customTicks = new System.Diagnostics.Stopwatch();
			var nativeTicks = new System.Diagnostics.Stopwatch();

			var ints = GenerateUInt64(PerfomanceRuns);

			nativeTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				ints[i].ToString(InvariantFormat);
			}
			nativeTicks.Stop();

			customTicks.Start();
			for (var i = 0; i < ints.Length; i++)
			{
				JsonUtils.UInt64ToBuffer(ints[i], buffer, 0, InvariantFormat);
			}
			customTicks.Stop();

			if (customTicks.ElapsedTicks <= nativeTicks.ElapsedTicks)
				Console.Write("perfomance boost x{0:F2}", (double)nativeTicks.ElapsedTicks / (double)customTicks.ElapsedTicks);
			else
				Console.Write("perfomance degrade x{0:F2}", (double)customTicks.ElapsedTicks / (double)nativeTicks.ElapsedTicks);
		}

		/// <summary>
		///A test for Unescape
		///</summary>
		[TestMethod()]
		public void UnescapeTest()
		{
			var testString = "qweqwe\\u0012\\u000D\\u000A\\u0007\\u0008'\\u0022\\u0022\\u0022\\u0007\\u0009\\u000Bawqwe123459абырв�алг";
			char[] charsToUnescape = testString.ToCharArray();
			int start = 1;
			int len = charsToUnescape.Length - 4;
			var expected = "weqwe\u0012\r\n\a\b\'\"\"\"\a\t\vawqwe123459абырв�";
			string actual;
			actual = JsonUtils.UnescapeBuffer(charsToUnescape, start, len);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		///A test for Escape
		///</summary>
		[TestMethod()]
		public void EscapeTest()
		{
			string stringToEscape = "qweqwe\u0012\r\n\a\b\'\"\"\"\a\t\vawqwe123459\\абырв�алг";
			char[] charsToEscape = stringToEscape.ToCharArray();
			char[] outputBuff = new char[1024];
			char[] tmpBuff = new char[1024];
			int start = 1;
			int len = charsToEscape.Length - 2;
			string expected = "weqwe\\u0012\\u000D\\u000A\\u0007\\u0008'\\u0022\\u0022\\u0022\\u0007\\u0009\\u000Bawqwe123459\\u005Cабырв�ал";
			string actual;
			actual = new string(outputBuff, 0, Math.Min(JsonUtils.EscapeBuffer(stringToEscape, start, outputBuff, 0, tmpBuff), expected.Length));

			Assert.AreEqual(expected, actual);
		}
	}
}
