﻿using Sencha.Serialization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace Sencha.Serialization.Tests
{


	/// <summary>
	///This is a test class for TypeConvertTest and is intended
	///to contain all TypeConvertTest Unit Tests
	///</summary>
	[TestClass()]
	public class TypeConvertTest
	{
		private TestContext testContextInstance;

		public TestContext TestContext
		{
			get
			{
				return testContextInstance;
			}
			set
			{
				testContextInstance = value;
			}
		}

		public void ConvertTestHelper<ResultT, SourceT>(SourceT source, ResultT expected)
		{
			IFormatProvider format = null;
			ResultT actual;

			actual = TypeConvert.Convert<SourceT, ResultT>(source, format);
			Assert.AreEqual(expected, actual);
		}
		public void WeakConvertTestHelper<ResultT, SourceT>(SourceT source, ResultT expected)
		{
			IFormatProvider format = null;
			object actual;
			actual = TypeConvert.Convert(typeof(ResultT), source, typeof(SourceT), format);

			if (actual != null)
				Assert.IsInstanceOfType(actual, typeof(ResultT));
			Assert.AreEqual(expected, actual);
		}

		[TestMethod()]
		public void CombinedNegativeConvertionTest()
		{
			// null to value
			try { ConvertTestHelper((int?)null, (byte)100); Assert.Fail("Should fail with exception"); }
			catch (ArgumentException) { }
			try { ConvertTestHelper((object)null, (byte)100); Assert.Fail("Should fail with exception"); }
			catch (ArgumentException) { }
			// overflow
			try { ConvertTestHelper((int)10000, (byte)100); Assert.Fail("Should fail with exception"); }
			catch (OverflowException) { }
			try { ConvertTestHelper("10000", (byte)100); Assert.Fail("Should fail with exception"); }
			catch (OverflowException) { }
			// wrong cast
			try { ConvertTestHelper(new object(), (byte)100); Assert.Fail("Should fail with exception"); }
			catch (InvalidCastException) { }
			try { ConvertTestHelper(6, Guid.NewGuid()); Assert.Fail("Should fail with exception"); }
			catch (InvalidCastException) { }
			try { ConvertTestHelper(default(System.TimeSpan), (byte)0); Assert.Fail("Should fail with exception"); }
			catch(InvalidCastException) { }
			// wrong format
			try { ConvertTestHelper("aaa", 100); Assert.Fail("Should fail with exception"); }
			catch (FormatException) { }
			try { ConvertTestHelper("NonColor", ConsoleColor.Cyan); Assert.Fail("Should fail with exception"); }
			catch (FormatException) { }
			try { ConvertTestHelper("a", new Version(10, 0, 0, 1)); Assert.Fail("Should fail with exception"); }
			catch (ArgumentException) { }
		}

		[TestMethod()]
		public void CombinedWeakConvertionTest()
		{
			// weak to nulls
			WeakConvertTestHelper((byte)100, (int?)100);
			WeakConvertTestHelper((byte)100, (int)100);
			WeakConvertTestHelper((byte?)100, (int)100);
			WeakConvertTestHelper((byte?)100, (int?)100);
			WeakConvertTestHelper((byte?)null, (int?)null);

			// 
			WeakConvertTestHelper((byte)byte.MaxValue, (short)byte.MaxValue);
			WeakConvertTestHelper((short)short.MaxValue, (int)short.MaxValue);
			WeakConvertTestHelper((int)int.MaxValue, (long)int.MaxValue);
			WeakConvertTestHelper((float)short.MaxValue, (double)short.MaxValue);
			WeakConvertTestHelper((float)short.MaxValue, (decimal)short.MaxValue);
			// 
			WeakConvertTestHelper((byte)byte.MinValue, (short)byte.MinValue);
			WeakConvertTestHelper((short)short.MinValue, (int)short.MinValue);
			WeakConvertTestHelper((int)int.MinValue, (long)int.MinValue);
			WeakConvertTestHelper((float)short.MinValue, (double)short.MinValue);
			WeakConvertTestHelper((float)short.MinValue, (decimal)short.MinValue);
			// 
			WeakConvertTestHelper((short)byte.MaxValue, (byte)byte.MaxValue);
			WeakConvertTestHelper((int)short.MaxValue, (short)short.MaxValue);
			WeakConvertTestHelper((long)int.MaxValue, (int)int.MaxValue);
			WeakConvertTestHelper((double)short.MaxValue, (float)short.MaxValue);
			WeakConvertTestHelper((decimal)short.MaxValue, (float)short.MaxValue);
			// 
			WeakConvertTestHelper((short)byte.MinValue, (byte)byte.MinValue);
			WeakConvertTestHelper((int)short.MinValue, (short)short.MinValue);
			WeakConvertTestHelper((long)int.MinValue, (int)int.MinValue);
			WeakConvertTestHelper((double)short.MinValue, (float)short.MinValue);
			WeakConvertTestHelper((decimal)short.MinValue, (float)short.MinValue);

			// string to number
			WeakConvertTestHelper("1", (byte)1);
			WeakConvertTestHelper("123", (short)123);
			WeakConvertTestHelper("12345", (int)12345);
			WeakConvertTestHelper("123456789", (long)123456789);
			WeakConvertTestHelper("123456789", (float)123456789);
			WeakConvertTestHelper("123456789", (double)123456789);
			WeakConvertTestHelper("123456789", (decimal)123456789);
			// number to string
			WeakConvertTestHelper((byte)1, "1");
			WeakConvertTestHelper((short)123, "123");
			WeakConvertTestHelper((int)12345, "12345");
			WeakConvertTestHelper((long)123456789, "123456789");
			WeakConvertTestHelper((float)1234, "1234");
			WeakConvertTestHelper((double)1234, "1234");
			WeakConvertTestHelper((decimal)1234, "1234");

			// convertible to string and back
			WeakConvertTestHelper("1.0.0.1", new Version(1, 0, 0, 1));
			WeakConvertTestHelper(new Version(1, 0, 0, 1), "1.0.0.1");

			// enum to string and back
			WeakConvertTestHelper(ConsoleColor.Black, "Black");
			WeakConvertTestHelper("Black", ConsoleColor.Black);
			// enum to number
			WeakConvertTestHelper(ConsoleColor.DarkYellow, (byte)ConsoleColor.DarkYellow);
			WeakConvertTestHelper(ConsoleColor.DarkYellow, (int)ConsoleColor.DarkYellow);
			WeakConvertTestHelper(ConsoleColor.DarkYellow, (float)ConsoleColor.DarkYellow);
			// enum to enum
			WeakConvertTestHelper(System.DateTimeKind.Unspecified, ConsoleColor.Black);
		}

		/// <summary>
		/// Convert between types
		/// </summary>
		[TestMethod()]
		public void ConvertEnumToInt32Test()
		{
			ConvertTestHelper(ConsoleColor.DarkYellow, (byte)ConsoleColor.DarkYellow);
		}

		[TestMethod()]
		public void ConvertEnumToStringTest()
		{
			ConvertTestHelper(ConsoleColor.DarkYellow, ConsoleColor.DarkYellow.ToString());
		}

		[TestMethod()]
		public void ConvertByteToInt32Test()
		{
			ConvertTestHelper((byte)1, (int)1);
		}


		[TestMethod()]
		public void ConvertVersionToStringTest()
		{
			ConvertTestHelper(new Version(1, 0, 0, 1), "1.0.0.1");
		}

		[TestMethod()]
		public void ConvertSByteToInt32Test()
		{
			ConvertTestHelper((sbyte)-1, (int)-1);
		}

		[TestMethod()]
		public void ConvertInt16ToInt32Test()
		{
			ConvertTestHelper((short)-12323, (int)-12323);
		}

		[TestMethod()]
		public void ConvertUInt16ToInt32Test()
		{
			ConvertTestHelper((ushort)12423, (int)12423);
		}

		[TestMethod()]
		public void ConvertInt32ToInt32Test()
		{
			ConvertTestHelper((int)-1992423, (int)-1992423);
		}

		[TestMethod()]
		public void ConvertUInt32ToInt32Test()
		{
			ConvertTestHelper((uint)1992423, (int)1992423);
		}

		[TestMethod()]
		public void ConvertInt64ToInt32Test()
		{
			ConvertTestHelper((long)111992423, (int)111992423);
		}

		[TestMethod()]
		public void ConvertUInt64ToInt32Test()
		{
			ConvertTestHelper((ulong)111992423, (int)111992423);
		}

		[TestMethod()]
		public void ConvertSingleToInt32Test()
		{
			ConvertTestHelper((float)1992423, (int)1992423);
		}

		[TestMethod()]
		public void ConvertDoubleToInt32Test()
		{
			ConvertTestHelper((double)111992423, (int)111992423);
		}

		[TestMethod()]
		public void ConvertDecimalToInt32Test()
		{
			ConvertTestHelper((decimal)111992423, (int)111992423);
		}

		/// <summary>
		/// Convert between types
		/// </summary>
		[TestMethod()]
		public void ConvertStringToVersionTest()
		{
			ConvertTestHelper("1.0.0.1", new Version(1, 0, 0, 1));
		}

		[TestMethod()]
		public void ConvertInt8ToEnumTest()
		{
			ConvertTestHelper((byte)ConsoleColor.DarkYellow, ConsoleColor.DarkYellow);
		}

		[TestMethod()]
		public void ConvertStringToEnumTest()
		{
			ConvertTestHelper(ConsoleColor.DarkYellow.ToString(), ConsoleColor.DarkYellow);
		}

		[TestMethod()]
		public void ConvertStringToByteTest()
		{
			ConvertTestHelper("1", (byte)1);
		}

		[TestMethod()]
		public void ConvertStringToSByteTest()
		{
			ConvertTestHelper("-1", (sbyte)-1);
		}

		[TestMethod()]
		public void ConvertStringToInt16Test()
		{
			ConvertTestHelper("-23111", (short)-23111);
		}

		[TestMethod()]
		public void ConvertStringToUInt16Test()
		{
			ConvertTestHelper("45000", (ushort)45000);
		}

		[TestMethod()]
		public void ConvertStringToInt32Test()
		{
			ConvertTestHelper("-2000005000", (int)-2000005000);
		}

		[TestMethod()]
		public void ConvertStringToUInt32Test()
		{
			ConvertTestHelper("4000005000", (uint)4000005000);
		}

		[TestMethod()]
		public void ConvertStringToInt64Test()
		{
			ConvertTestHelper("-20000005000", (long)-20000005000);
		}

		[TestMethod()]
		public void ConvertStringToUInt64Test()
		{
			ConvertTestHelper("40000005000", (ulong)40000005000);
		}

		[TestMethod()]
		public void ConvertStringToDoubleTest()
		{
			ConvertTestHelper("-200.05", (double)-200.05);
		}

		[TestMethod()]
		public void ConvertStringToSingleTest()
		{
			ConvertTestHelper("200.05", (float)200.05);
		}

		[TestMethod()]
		public void ConvertStringToDecimalTest()
		{
			ConvertTestHelper("200.05", (decimal)200.05);
		}


		/// <summary>
		/// Convert to string
		/// </summary>
		[TestMethod()]
		public void ConvertByteToStringTest()
		{
			ConvertTestHelper((byte)1, "1");
		}

		[TestMethod()]
		public void ConvertSByteToStringTest()
		{
			ConvertTestHelper((sbyte)-1, "-1");
		}

		[TestMethod()]
		public void ConvertInt16ToStringTest()
		{
			ConvertTestHelper((short)-23111, "-23111");
		}

		[TestMethod()]
		public void ConvertUInt16ToStringTest()
		{
			ConvertTestHelper((ushort)45000, "45000");
		}

		[TestMethod()]
		public void ConvertInt32ToStringTest()
		{
			ConvertTestHelper((int)-2000005000, "-2000005000");
		}

		[TestMethod()]
		public void ConvertUInt32ToStringTest()
		{
			ConvertTestHelper((uint)4000005000, "4000005000");
		}

		[TestMethod()]
		public void ConvertInt64ToStringTest()
		{
			ConvertTestHelper((long)-2000005000, "-2000005000");
		}

		[TestMethod()]
		public void ConvertUInt64ToStringTest()
		{
			ConvertTestHelper((ulong)40000005000, "40000005000");
		}

		[TestMethod()]
		public void ConvertDoubleToStringTest()
		{
			ConvertTestHelper((double)-200.05, "-200.05");
		}

		[TestMethod()]
		public void ConvertSingleToStringTest()
		{
			ConvertTestHelper((float)200.05, "200.05");
		}

		[TestMethod()]
		public void ConvertDecimalToStringTest()
		{
			ConvertTestHelper((decimal)200.05, "200.05");
		}

		/// <summary>
		///A test for ConvertFromBuffer
		///</summary>
		public void ConvertFromBufferTestHelper<ResultT>(string str, ResultT expected)
		{
			char[] buffer = str.ToCharArray();
			int start = 0;
			int length = buffer.Length;
			IFormatProvider format = null;
			ResultT actual;
			actual = JsonUtils.ConvertFromBuffer<ResultT>(buffer, start, length, format);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		/// Convert from buffer
		/// </summary>
		[TestMethod()]
		public void ConvertBufferToByteTest()
		{
			ConvertFromBufferTestHelper("1", (byte)1);
		}

		[TestMethod()]
		public void ConvertBufferToSByteTest()
		{
			ConvertFromBufferTestHelper("-1", (sbyte)-1);
		}

		[TestMethod()]
		public void ConvertBufferToInt16Test()
		{
			ConvertFromBufferTestHelper("-23111", (short)-23111);
		}

		[TestMethod()]
		public void ConvertBufferToUInt16Test()
		{
			ConvertFromBufferTestHelper("45000", (ushort)45000);
		}

		[TestMethod()]
		public void ConvertBufferToInt32Test()
		{
			ConvertFromBufferTestHelper("-2000005000", (int)-2000005000);
		}

		[TestMethod()]
		public void ConvertBufferToUInt32Test()
		{
			ConvertFromBufferTestHelper("4000005000", (uint)4000005000);
		}

		[TestMethod()]
		public void ConvertBufferToInt64Test()
		{
			ConvertFromBufferTestHelper("-20000005000", (long)-20000005000);
		}

		[TestMethod()]
		public void ConvertBufferToUInt64Test()
		{
			ConvertFromBufferTestHelper("40000005000", (ulong)40000005000);
		}

		[TestMethod()]
		public void ConvertBufferToDoubleTest()
		{
			ConvertFromBufferTestHelper("-200.05", (double)-200.05);
		}

		[TestMethod()]
		public void ConvertBufferToSingleTest()
		{
			ConvertFromBufferTestHelper("200.05", (float)200.05);
		}

		[TestMethod()]
		public void ConvertBufferToDecimalTest()
		{
			ConvertFromBufferTestHelper("200.05", (decimal)200.05);
		}

		/// <summary>
		///A test for ConvertToBuffer
		///</summary>
		public void ConvertToBufferTestHelper<ResultT>(ResultT value, string expected)
		{
			char[] buffer = new char[1024];
			int start = 0;
			IFormatProvider format = null;
			int written = JsonUtils.ConvertToBuffer<ResultT>(buffer, start, value, format);
			var actual = new string(buffer, 0, written);
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		/// Convert to buffer
		/// </summary>
		[TestMethod()]
		public void ConvertByteToBufferTest()
		{
			ConvertToBufferTestHelper((byte)1, "1");
		}

		[TestMethod()]
		public void ConvertSByteToBufferTest()
		{
			ConvertToBufferTestHelper((sbyte)-1, "-1");
		}

		[TestMethod()]
		public void ConvertInt16ToBufferTest()
		{
			ConvertToBufferTestHelper((short)-23111, "-23111");
		}

		[TestMethod()]
		public void ConvertUInt16ToBufferTest()
		{
			ConvertToBufferTestHelper((ushort)45000, "45000");
		}

		[TestMethod()]
		public void ConvertInt32ToBufferTest()
		{
			ConvertToBufferTestHelper((int)-2000005000, "-2000005000");
		}

		[TestMethod()]
		public void ConvertUInt32ToBufferTest()
		{
			ConvertToBufferTestHelper((uint)4000005000, "4000005000");
		}

		[TestMethod()]
		public void ConvertInt64ToBufferTest()
		{
			ConvertToBufferTestHelper((long)-2000005000, "-2000005000");
		}

		[TestMethod()]
		public void ConvertUInt64ToBufferTest()
		{
			ConvertToBufferTestHelper((ulong)40000005000, "40000005000");
		}

		[TestMethod()]
		public void ConvertDoubleToBufferTest()
		{
			ConvertToBufferTestHelper((double)-200.05, "-200.05");
		}

		[TestMethod()]
		public void ConvertSingleToBufferTest()
		{
			ConvertToBufferTestHelper((float)200.05, "200.05");
		}

		[TestMethod()]
		public void ConvertDecimalToBufferTest()
		{
			ConvertToBufferTestHelper((decimal)200.05, "200.05");
		}
	}
}
