﻿using System;
using Sencha.Serialization;
using System.Collections.Generic;

namespace Serialization.Serialization.IntegrationTests
{
	public class BigPlainObject
	{
		public enum MyEnum
		{
			HereIsMyEnumValue = 100,
			AnotherValue = -100
		}

		public byte? ByteField;
		public sbyte SByteField;
		public Int16 Int16Field;
		public UInt16 UInt16Field;
		public Int32 Int32Field;
		public UInt32 UInt32Field;
		public Int64 Int64Field;
		public UInt64 UInt64Field;
		public Decimal DecimalField;
		public Single SingleFiled;
		public Double DoubleField;
		public String StringField;
		public MyEnum MyEnumField;
		public bool BooleanField;
		public BigPlainObject NestedObject1;
		public DateTime DateTimeField;
		public DateTimeOffset DateTimeOffsetField;
		public Guid GuidField;
		public TimeSpan TimeSpanField;
		public Uri UrlField;
		public string NullField;
		public int[] IntArrayField;
		public string[] StringArrayField;
		public int[] EmptyArrayField;
		public int[] NullArrayField;
		public Dictionary<string, object>[] DictionaryArrayField;
		public IList<object> ObjectArrayField;
		public KeyValue<int, string> FieldWithVeryVeryLongNameForField;
		public List<BigPlainObject> NestedObjectArrayField;

		public BigPlainObject()
		{

		}
		public BigPlainObject(bool fill)
		{
			ByteField = 1;
			SByteField = -1;
			Int16Field = -566;
			UInt16Field = 622;
			Int32Field = -123123123;
			UInt32Field = 123123123;
			Int64Field = -123213213123123;
			UInt64Field = 12123123123213;
			DecimalField = 132323123123123123.123123213123m;
			SingleFiled = 123123.123213f;
			DoubleField = 123213.123123123;
			StringField = "Here is my String";
			MyEnumField = MyEnum.HereIsMyEnumValue;
			BooleanField = true;
			DateTimeField = DateTime.ParseExact("2010-10-30T14:25:55.391+04:00", Json.DateTimeFormats[0], Json.DefaultFormatProvider);
			DateTimeOffsetField = DateTime.ParseExact("2010-10-30T14:25:55.391+04:00", Json.DateTimeFormats[0], Json.DefaultFormatProvider);
			GuidField = new Guid("{7054773A-741D-4FCF-B002-5252EC93E8E3}");
			TimeSpanField = TimeSpan.Parse("1:00:00");
			UrlField = new Uri("http://www.example.com/");
			NullField = null;
			IntArrayField = new int[] { 1, 2, 3 };
			StringArrayField = new string[] { "a", "B", "c" };
			EmptyArrayField = new int[0];
			NullArrayField = null;
			DictionaryArrayField = new Dictionary<string, object>[] { new Dictionary<string, object>(), new Dictionary<string, object>(), new Dictionary<string, object>() };
			ObjectArrayField = new List<object>(new object[] { new object(), new object(), new object() });
			FieldWithVeryVeryLongNameForField = new KeyValue<int, string>(1, "100");
		}
		public BigPlainObject(int seed, int maxDepth, ref long maxObjects)
		{
			var random = new Random(seed);

			ByteField = (byte)random.Next(byte.MinValue, byte.MaxValue);
			SByteField = (sbyte)random.Next(sbyte.MinValue, sbyte.MaxValue);
			Int16Field = (short)random.Next(short.MinValue, short.MaxValue);
			UInt16Field = (ushort)random.Next(ushort.MinValue, ushort.MaxValue);
			Int32Field = (int)random.Next(int.MinValue, int.MaxValue);
			UInt32Field = (uint)random.Next(int.MinValue, int.MaxValue);
			Int64Field = (long)(((ulong)(uint)random.Next(int.MinValue, int.MaxValue) << 32) | (uint)random.Next(int.MinValue, int.MaxValue));
			UInt64Field = (((ulong)(uint)random.Next(int.MinValue, int.MaxValue) << 32) | (uint)random.Next(int.MinValue, int.MaxValue));
			DecimalField = (decimal)random.NextDouble() * decimal.MaxValue;
			SingleFiled = (float)random.NextDouble() * float.MaxValue;
			DoubleField = (double)random.NextDouble() * double.MaxValue;
			var chars = random.Next(0, 100) > 50 ? new char[random.Next(0, 80000)] : null;
			if (chars != null) for (var i = 0; i < chars.Length; i++) chars[i] = (char)random.Next('A', 'z');
			StringField = chars != null ? new string(chars, 0, chars.Length) : null;
			MyEnumField = (MyEnum)random.Next();
			BooleanField = random.Next(0, 100) > 50;
			DateTimeField = new DateTime((long)(DateTime.MaxValue.Ticks * random.NextDouble()));
			DateTimeOffsetField = new DateTimeOffset(new DateTime((long)(DateTime.MaxValue.Ticks * random.NextDouble())), TimeSpan.FromHours(random.Next(0, 14)));
			GuidField = new Guid((uint)random.Next(), (ushort)random.Next(ushort.MinValue, ushort.MaxValue), (ushort)random.Next(ushort.MinValue, ushort.MaxValue), (byte)random.Next(byte.MinValue, byte.MaxValue), (byte)random.Next(byte.MinValue, byte.MaxValue), (byte)random.Next(byte.MinValue, byte.MaxValue), (byte)random.Next(byte.MinValue, byte.MaxValue), (byte)random.Next(byte.MinValue, byte.MaxValue), (byte)random.Next(byte.MinValue, byte.MaxValue), (byte)random.Next(byte.MinValue, byte.MaxValue), (byte)random.Next(byte.MinValue, byte.MaxValue));
			TimeSpanField = new TimeSpan((long)(((ulong)(uint)random.Next(int.MinValue, int.MaxValue) << 32) | (uint)random.Next(int.MinValue, int.MaxValue)));
			UrlField = random.Next(0, 100) > 50 ? new Uri("tcp://hellloooo.com/" + Uri.EscapeUriString(Convert.ToBase64String(BitConverter.GetBytes(random.NextDouble())))) : null;
			NullField = null;
			IntArrayField = random.Next(0, 100) > 50 ? new int[random.Next(10000)] : null;
			if (IntArrayField != null) for (var i = 0; i < IntArrayField.Length; i++) IntArrayField[i] = random.Next();
			StringArrayField = random.Next(0, 100) > 50 ? new string[random.Next(1000)] : null;
			if (StringArrayField != null) for (var i = 0; i < StringArrayField.Length; i++) StringArrayField[i] = BitConverter.ToString(BitConverter.GetBytes(random.NextDouble()));
			EmptyArrayField = new int[0];
			NullArrayField = null;
			DictionaryArrayField = random.Next(0, 100) > 50 ? new Dictionary<string, object>[random.Next(100)] : null;
			if (DictionaryArrayField != null) for (var i = 0; i < DictionaryArrayField.Length; i++) DictionaryArrayField[i] = new Dictionary<string, object>();
			ObjectArrayField = random.Next(0, 100) > 50 ? new List<object>(new object[random.Next(100)]) : null;
			if (ObjectArrayField != null) for (var i = 0; i < ObjectArrayField.Count; i++) ObjectArrayField[i] = new object();
			FieldWithVeryVeryLongNameForField = new KeyValue<int, string>(random.Next(), random.Next(0, 1000) > 50 ? Convert.ToBase64String(BitConverter.GetBytes(random.NextDouble())) : null);

			if (StringField != null) maxObjects--;
			if (UrlField != null) maxObjects--;
			if (IntArrayField != null) maxObjects -= IntArrayField.Length + 1;
			if (StringArrayField != null) maxObjects -= StringArrayField.Length + 1;
			if (EmptyArrayField != null) maxObjects--;
			if (DictionaryArrayField != null) maxObjects -= DictionaryArrayField.Length + 1;
			if (ObjectArrayField != null) maxObjects -= ObjectArrayField.Count + 1;
			if (FieldWithVeryVeryLongNameForField != null) maxObjects--;
			maxDepth--;

			if (maxObjects <= 0 || maxDepth <= 0)
				return;

			NestedObject1 = new BigPlainObject(random.Next(), maxDepth, ref maxObjects);

			if (maxObjects <= 0)
				return;

			NestedObjectArrayField = new List<BigPlainObject>(new BigPlainObject[random.Next(10, (int)Math.Min(maxObjects / 100, 1000))]);
			for (int i = 0; i < NestedObjectArrayField.Count; i++)
				NestedObjectArrayField[i] = random.Next(0, 100) > 20 ? new BigPlainObject(random.Next(), maxDepth, ref maxObjects) : null;
		}

		public override int GetHashCode()
		{
			return base.GetHashCode();
		}

		public override bool Equals(object obj)
		{
			var other = obj as BigPlainObject;

			if (other == null)
				return false;

			if (!Object.Equals(this.ByteField, other.ByteField)) return false;
			if (!Object.Equals(this.SByteField, other.SByteField)) return false;
			if (!Object.Equals(this.Int16Field, other.Int16Field)) return false;
			if (!Object.Equals(this.UInt16Field, other.UInt16Field)) return false;
			if (!Object.Equals(this.Int32Field, other.Int32Field)) return false;
			if (!Object.Equals(this.UInt32Field, other.UInt32Field)) return false;
			if (!Object.Equals(this.Int64Field, other.Int64Field)) return false;
			if (!Object.Equals(this.UInt64Field, other.UInt64Field)) return false;
			if (!Object.Equals(this.DecimalField, other.DecimalField)) return false;
			if (!Object.Equals(this.SingleFiled, other.SingleFiled)) return false;
			if (!Object.Equals(this.DoubleField, other.DoubleField)) return false;
			if (!Object.Equals(this.StringField, other.StringField)) return false;
			if (!Object.Equals(this.MyEnumField, other.MyEnumField)) return false;
			if (!Object.Equals(this.BooleanField, other.BooleanField)) return false;
			if (!Object.Equals(this.NestedObject1, other.NestedObject1)) return false;
			if (!Object.Equals(this.DateTimeField, other.DateTimeField)) return false;
			if (!Object.Equals(this.DateTimeOffsetField, other.DateTimeOffsetField)) return false;
			if (!Object.Equals(this.GuidField, other.GuidField)) return false;
			if (!Object.Equals(this.TimeSpanField, other.TimeSpanField)) return false;
			if (!Object.Equals(this.UrlField, other.UrlField)) return false;
			if (!Object.Equals(this.NullField, other.NullField)) return false;
			if (!StructEquals.Are(this.IntArrayField, other.IntArrayField)) return false;
			if (!StructEquals.Are(this.EmptyArrayField, other.EmptyArrayField)) return false;
			if (!StructEquals.Are(this.NullArrayField, other.NullArrayField)) return false;
			if (!StructEquals.Are(this.DictionaryArrayField, other.DictionaryArrayField, StructEquals.BothNotNullOrNull)) return false;
			if (!StructEquals.Are(this.ObjectArrayField, other.ObjectArrayField, StructEquals.BothNotNullOrNull)) return false;
			if (!Object.Equals(this.FieldWithVeryVeryLongNameForField, other.FieldWithVeryVeryLongNameForField)) return false;

			return true;
		}
	}
}
