﻿using System;
using Sencha.Serialization.Contracts;
using Sencha.Serialization;
using System.Diagnostics;
using System.Collections.Generic;
using System.Collections;

namespace Serialization.Serialization.IntegrationTests
{
    static class ReadJson
    {
        public static void FromStream<T>(T expected, string resourceName)
        {
            var serializableType = typeof(T);
            var interpreter = MetadataInterpreter.DataContract;
            var descriptor = MetadataDescriber.ReflectionPublicOnly;
            var contract = new DataContract(serializableType, interpreter, descriptor);
			var target = new JsonSerializer(contract) { IgnoreCustomSerialization = false, SerializeByteArrayAsBase64 = true, SerializeEnumsAsInteger = false, SkipUnaccessibleMembers = true, SkipUnknownMember = true, StringInterchange = true, ArrayInterchange = true };
            using (var jsonStream = Resources.GetStream(resourceName))
            {

                var actual = target.ReadObject<T>(jsonStream);

                Debug.Assert(Object.Equals(expected, actual), string.Format("object are not equals: expected={0}, actual={1}", expected, actual));
            }
        }
        public static void FromStream<T>(T[] expected, string resourceName)
        {
            var serializableType = typeof(T[]);
            var interpreter = MetadataInterpreter.DataContract;
            var descriptor = MetadataDescriber.ReflectionPublicOnly;
            var contract = new DataContract(serializableType, interpreter, descriptor);
			var target = new JsonSerializer(contract) { IgnoreCustomSerialization = false, SerializeByteArrayAsBase64 = false, SerializeEnumsAsInteger = false, SkipUnaccessibleMembers = true, SkipUnknownMember = true, StringInterchange = true, ArrayInterchange = true };
            using (var jsonStream = Resources.GetStream(resourceName))
            {

                var actual = target.ReadObject<T[]>(jsonStream);

                if (Object.ReferenceEquals(expected, actual)) // are null
                    return;

                for (var i = 0; i < expected.Length; i++)
                {
                    Debug.Assert(Object.Equals(expected[i], actual[i]), string.Format("array's items at [{0}] are not equals expected={2}, actual={1}", i, actual[i], expected[i]));
                }
            }
        }
        public static void FromStream<T>(List<T> expected, string resourceName)
        {
            var serializableType = typeof(IList<T>);
            var interpreter = MetadataInterpreter.DataContract;
            var descriptor = MetadataDescriber.ReflectionPublicOnly;
            var contract = new DataContract(serializableType, interpreter, descriptor);
			var target = new JsonSerializer(contract) { IgnoreCustomSerialization = false, SerializeByteArrayAsBase64 = true, SerializeEnumsAsInteger = false, SkipUnaccessibleMembers = true, SkipUnknownMember = true, StringInterchange = true, ArrayInterchange = true };
            using (var jsonStream = Resources.GetStream(resourceName))
            {

                var actual = target.ReadObject<List<T>>(jsonStream);

                if (Object.ReferenceEquals(expected, actual)) // are null
                    return;

                for (var i = 0; i < expected.Count; i++)
                {
                    Debug.Assert(Object.Equals(expected[i], actual[i]), string.Format("list's items at [{0}] are not equals expected={2}, actual={1}", i, actual[i], expected[i]));
                }
            }
        }
        public static void FromStream<KeyT, ValueT>(Dictionary<KeyT, ValueT> expected, string resourceName)
        {
            var serializableType = typeof(Dictionary<KeyT, ValueT>);
            var interpreter = MetadataInterpreter.DataContract;
            var descriptor = MetadataDescriber.ReflectionPublicOnly;
            var contract = new DataContract(serializableType, interpreter, descriptor);
			var target = new JsonSerializer(contract) { IgnoreCustomSerialization = false, SerializeByteArrayAsBase64 = true, SerializeEnumsAsInteger = false, SkipUnaccessibleMembers = true, SkipUnknownMember = true, StringInterchange = true, ArrayInterchange = true };
            using (var jsonStream = Resources.GetStream(resourceName))
            {

                var actual = target.ReadObject<Dictionary<KeyT, ValueT>>(jsonStream);

                if (Object.ReferenceEquals(expected, actual)) // are null
                    return;

                foreach (var key in expected.Keys)
                {
                    Debug.Assert(Object.Equals(expected[key], actual[key]), string.Format("dictionary's items at [{0}] are not equals expected={2}, actual={1}", key, actual[key], expected[key]));
                }
            }
        }
        public static void FromStream(Hashtable expected, string resourceName)
        {
            var serializableType = typeof(IDictionary);
            var interpreter = MetadataInterpreter.DataContract;
            var descriptor = MetadataDescriber.ReflectionPublicOnly;
            var contract = new DataContract(serializableType, interpreter, descriptor);
			var target = new JsonSerializer(contract) { IgnoreCustomSerialization = false, SerializeByteArrayAsBase64 = true, SerializeEnumsAsInteger = false, SkipUnaccessibleMembers = true, SkipUnknownMember = true, StringInterchange = true, ArrayInterchange = true };
            using (var jsonStream = Resources.GetStream(resourceName))
            {

                var actual = target.ReadObject<IDictionary>(jsonStream);

                if (Object.ReferenceEquals(expected, actual)) // are null
                    return;

                foreach (var key in expected.Keys)
                {
                    Debug.Assert(Object.Equals(expected[key], actual[key]), string.Format("dictionary's items at [{0}] are not equals expected={2}, actual={1}", key, actual[key], expected[key]));
                }
            }
        }
        public static void FromFile<T>(T expected, string fileName)
        {
            var serializableType = typeof(T);
            var interpreter = MetadataInterpreter.DataContract;
            var descriptor = MetadataDescriber.ReflectionPublicOnly;
            var contract = new DataContract(serializableType, interpreter, descriptor);
			var target = new JsonSerializer(contract) { IgnoreCustomSerialization = false, SerializeByteArrayAsBase64 = true, SerializeEnumsAsInteger = false, SkipUnaccessibleMembers = true, SkipUnknownMember = true, StringInterchange = true, ArrayInterchange = true };
            using (var jsonStream = System.IO.File.OpenRead(fileName))
            {

                var actual = target.ReadObject<T>(jsonStream);

                Debug.Assert(Object.Equals(expected, actual));
            }
        }
    }
}
