﻿using System;
using System.Collections.Generic;

namespace Serialization.Serialization.IntegrationTests
{
	static class StructEquals
	{
		internal static bool Are<T1>(IList<T1> a1, IList<T1> a2, Func<object, object, bool> valueComparer = null)
		{
			if (Object.ReferenceEquals(a1, a2))
				return true;

			if (a1.Count != a2.Count)
				return false;

			for (int i = 0; i < a1.Count; i++)
				if (valueComparer != null ? !valueComparer(a1[i], a2[i]) : !Object.Equals(a1[i], a2[i]))
					return false;

			return true;
		}
		internal static bool Are<K, V>(IDictionary<K, V> a, IDictionary<K, V> b, Func<object, object, bool> valueComparer)
		{
			if (Object.ReferenceEquals(a, b))
				return true;

			if (a.Count != b.Count)
				return false;

			foreach (var key in a.Keys)
			{
				if (!b.ContainsKey(key))
					return false;

				if (!valueComparer(a[key], b[key]))
					return false;
			}

			return true;
		}

		internal static bool BothNotNullOrNull<T>(T a, T b)
		{
			return ReferenceEquals(a, b) || (a != null && b != null); // both null or not null
		}
	}
}
